//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function RpEngine()
{
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

RpEngine.CURRENT_STATE_GAME = 'game';
RpEngine.CURRENT_STATE_TITLE = 'title';

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

RpEngine.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------
	
	_currentState : '',
		
	config : null,
	logger : null,
	util : null,
		
	assetManager : null,
	langManager : null,
	moduleManager : null,
	soundEngine : null,
	
	titleScreen : null,
	gameScreen : null,
		
	game : null,

	//------------------------------------------------------------------------------------------------------------
	//INITIALIZERS - SERIALIZERS
	//------------------------------------------------------------------------------------------------------------

	init : function()
	{
		$('#gameScreen').hide();
		$('#titleScreen').hide();
		
		this.util = new rp.Util();
		this.logger = new rp.Logger();
		this.config = new rp.Config();
		
		this.assetManager = new rp.AssetManager();
		this.langManager = new rp.LangManager();
		this.moduleManager = new rp.ModuleManager();
		this.soundEngine = new rp.SoundEngine();

		this.titleScreen = new rp.TitleScreen();
		this.gameScreen = new rp.GameScreen();

		this.moduleManager.loadModuleData();
		
		this.showTitleScreen();
		
		this.soundEngine.playMusic(this.moduleManager.getCurrentData().getDefaultData(rp.Module.DEFAULT_DATA_TITLE_MUSIC_ID));

		rpEngine.logger.log('RolePlanez engine started successfully.', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	_loadGameSub : function(filePath)
	{
		var data = rp.Util.loadFileJson(filePath);
		
		if (data == null)
		{
			rpEngine.logger.log('RpEngine._loadGameSub -- Error loading game from "%s".', [filePath], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
			return;
		}

		this.gameScreen.resetGui();
		this.game = new rp.Game();
		this.game.initGameLoad(data);

		this.showGameScreen();
		
		rpEngine.logger.log('RpEngine._loadGameSub -- Gamed loaded successfully from "%s".', [filePath], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
	},
	
	_saveGameSub : function(filePath)
	{
		var saveGameData = new rp.GameSave(this.game);
		var dataString = JSON.stringify(saveGameData, null, 4)
		
		if (rp.Util.saveFile(filePath, dataString, false) == true)
		{
			rpEngine.logger.log('RpEngine._saveGameSub -- Gamed saved successfully to "%s".', [filePath], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		}
		else
		{
			rpEngine.logger.log('RpEngine._saveGameSub -- Error saving game to "%s".', [filePath], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
		}
	},
	
	endGame : function()
	{
		rpEngine.logger.log('RpEngine.newGame -- Ending game...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		this.soundEngine.playMusic(this.moduleManager.getCurrentData().getDefaultData(rp.Module.DEFAULT_DATA_TITLE_MUSIC_ID));
		
		this.game = null;
		this.showTitleScreen();
	},
	
	loadGame : function()
	{
		rpEngine.logger.log('RpEngine.loadGame -- Loading game...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

		rp.Util.createDirectory(appBasePath + 'save');
		
		var defaultPath = (appBasePath + 'save');
		var filePathList = dialog.showOpenDialog(
				remote.getCurrentWindow(),
				{
					'title': 'Load game',
					'defaultPath': defaultPath,
					'filters': [{ 'name': 'RolePlanez savefile', 'extensions': ['rpz'] }]
				});

		if (filePathList == undefined)
		{
			rpEngine.logger.log('RpEngine.loadGame -- Cancelled loading game.', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
			return;
		}
		
		this._loadGameSub(filePathList[0]);
	},
	
	newGame : function()
	{
		rpEngine.logger.log('RpEngine.newGame -- Starting new game...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		this.soundEngine.stopMusic();
		
		this.game = new rp.Game();
		var result = this.game.initGameNew();
		if (result == false)
		{
			rpEngine.logger.log('RpEngine.newGame -- Error in "game.initGameNew", aborting new game.', [], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
			
			this.soundEngine.playMusic(this.moduleManager.getCurrentData().getDefaultData(rp.Module.DEFAULT_DATA_TITLE_MUSIC_ID));
			
			return;
		}

		rpEngine.gameScreen.resetGui();
		this.showGameScreen();
	},
	
	quickLoad : function()
	{
		rpEngine.logger.log('RpEngine.saveGame -- Quick loading game...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

		var filePath = (appBasePath + 'save/quick.rpz');
		
		this._loadGameSub(filePath);
	},
	
	quickSave : function()
	{
		if (this._currentState != RpEngine.CURRENT_STATE_GAME)
		{
			return;
		}
		
		rpEngine.logger.log('RpEngine.saveGame -- Quick saving game...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

		rp.Util.createDirectory(appBasePath + 'save');
		
		var filePath = (appBasePath + 'save/quick.rpz');
		
		this._saveGameSub(filePath);
	},
	
	quitGame : function()
	{
		rpEngine.logger.log('RpEngine.quitGame -- Quitting game.', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		remote.getCurrentWindow().close();
	},
	
	saveGame : function()
	{
		if (this._currentState != RpEngine.CURRENT_STATE_GAME)
		{
			return;
		}
		
		rpEngine.logger.log('RpEngine.saveGame -- Saving game...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

		rp.Util.createDirectory(appBasePath + 'save');
		
		var defaultPath = (appBasePath + 'save');
		var filePath = dialog.showSaveDialog(
				remote.getCurrentWindow(),
				{
					'title': 'Save game',
					'defaultPath': defaultPath,
					'filters': [{ 'name': 'RolePlanez savefile', 'extensions': ['rpz'] }]
				});
		
		if (filePath == undefined)
		{
			rpEngine.logger.log('RpEngine.saveGame -- Cancelled saving game.', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
			return;
		}
		
		this._saveGameSub(filePath);
	},
	
	showGameScreen : function()
	{
		this._currentState = RpEngine.CURRENT_STATE_GAME;
		
		$('#gameScreen').fadeIn();
		$('#titleScreen').fadeOut();
	},
	
	showTitleScreen : function()
	{
		this._currentState = RpEngine.CURRENT_STATE_TITLE;
		
		$('#gameScreen').fadeOut();
		$('#titleScreen').fadeIn();
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------
		
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = RpEngine;

//------------------------------------------------------------------------------------------------------------
