//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function AssetPack()
{
}
var AssetPack = AssetPack;

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

AssetPack.FILE_ROLE_VALUE = 'asset';

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

AssetPack.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_cssPathMap : {},
	_fontPathMap : {},
	_imagePathMap : {},
	_scriptPathMap : {},
	_soundPathMap : {},
	
	_scriptDataMap : {},
	
	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	loadData : function()
	{
		for (var id in this._scriptPathMap)
		{
			var filePath = this._scriptPathMap[id];

			rpEngine.logger.log('Loading script file "%s"...', [filePath], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

			var scriptData = null;
			try
			{
				scriptData = require(rpEngine.moduleManager.formatCurrentModuleFilePath(filePath));
			}
			catch (exception)
			{
			}
			
			if (scriptData == null)
			{
				rpEngine.logger.log('AssetPack.loadData -- Error, could not load script file "%s".', [filePath], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
				continue;
			}
			
			this.setScriptData(id, scriptData);
		}
	},
	
	loadFile : function(filePath)
	{
		rpEngine.logger.log('Loading asset file "%s"...', [filePath], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

		var filePathReal = appBasePath + filePath;
		var fileData = rp.Util.loadFileJson(filePathReal);
		if (fileData == null)
		{
			rpEngine.logger.log('AssetPack.loadFile -- Error, could not load asset file "%s".', [filePath], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
			return false;
		}

		//Vérification des erreurs
		
		if (fileData.role != AssetPack.FILE_ROLE_VALUE)
		{
			rpEngine.logger.log('AssetPack.loadFile -- Error, bad role ID for asset file "%s".', [filePath], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
			return false;
		}
		
		//Chargement des données

		for (var id in fileData.cssList)
		{
			this.setCssPath(id, fileData.cssList[id]); 
		}
		
		for (var id in fileData.fontList)
		{
			this.setFontPath(id, fileData.fontList[id]);
		}
		
		for (var id in fileData.imageList)
		{
			this.setImagePath(id, fileData.imageList[id]);
		}
		
		for (var id in fileData.scriptList)
		{
			this.setScriptPath(id, fileData.scriptList[id]);
		}
		
		for (var id in fileData.soundList)
		{
			this.setSoundPath(id, fileData.soundList[id]);
		}
		
		this.loadData();
		
		return true;
	},
		
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------

	getCssPath : function(id)
	{
		return this._cssPathMap[id];
	},
	getCssPathMap : function()
	{
		return this._cssPathMap;
	},
	setCssPath : function(id, value)
	{
		this._cssPathMap[id] = value;
	},

	getFontPath : function(id)
	{
		return this._fontPathMap[id];
	},
	getFontPathMap : function()
	{
		return this._fontPathMap;
	},
	setFontPath : function(id, value)
	{
		this._fontPathMap[id] = value;
	},
	
	getImagePath : function(id)
	{
		return this._imagePathMap[id];
	},
	getImagePathMap : function()
	{
		return this._imagePathMap;
	},
	setImagePath : function(id, value)
	{
		this._imagePathMap[id] = value;
	},
	
	getScriptData : function(id)
	{
		return this._scriptDataMap[id];
	},
	getScriptDataMap : function()
	{
		return this._scriptDataMap;
	},
	setScriptData : function(id, value)
	{
		this._scriptDataMap[id] = value;
	},
	
	getScriptPath : function(id)
	{
		return this._scriptPathMap[id];
	},
	getScriptPathMap : function()
	{
		return this._scriptPathMap;
	},
	setScriptPath : function(id, value)
	{
		this._scriptPathMap[id] = value;
	},

	getSoundPath : function(id)
	{
		return this._soundPathMap[id];
	},
	getSoundPathMap : function()
	{
		return this._soundPathMap;
	},
	setSoundPath : function(id, value)
	{
		this._soundPathMap[id] = value;
	},
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = AssetPack;

//------------------------------------------------------------------------------------------------------------
