//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function LangPack()
{
	this._stringMaps = {};
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

LangPack.FILE_ROLE_VALUE = 'lang';

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

LangPack.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_stringMaps : null, //Language ID => String ID => String value
		
	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	loadFile : function(filePath)
	{
		rpEngine.logger.log('Loading lang file "%s"...', [filePath], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

		var filePathReal = appBasePath + filePath;
		var fileData = rp.Util.loadFileJson(filePathReal);
		if (fileData == null)
		{
			rpEngine.logger.log('LangPack.loadFile -- Could not load lang file "%s".', [filePath], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
			
			return false;
		}

		//Vérification des erreurs
		
		if (fileData.role != rp.LangPack.FILE_ROLE_VALUE)
		{
			rpEngine.logger.log('LangPack.loadFile -- Bad role ID for lang file "%s".', [filePath], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
			return false;
		}

		if (fileData.lang == "")
		{
			rpEngine.logger.log('LangPack.loadFile -- No language ID found for lang file "%s".', [filePath], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
			return false;
		}
		
		//Chargement des données

		for (var stringId in fileData.stringList)
		{
			this.setStringWithLanguage(stringId, fileData.languageId, fileData.stringList[stringId]);
		}
		
		return true;
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------

	getString : function(id)
	{
		return this.getStringWithLanguage(id, rpEngine.langManager.getCurrentLanguageId());
	},
	
	getStringWithLanguage : function(stringId, languageId)
	{
		if (this._stringMaps[languageId] == null)
		{
			return '{Unknown - "' + languageId + '" - "' + stringId + '"}';
		}
		
		if (this._stringMaps[languageId][stringId] == null)
		{
			return '{Unknown - "' + languageId + '" - "' + stringId + '"}';
		}
		
		return this._stringMaps[languageId][stringId];
	},
	setStringWithLanguage : function(stringId, languageId, value)
	{
		if (this._stringMaps[languageId] == null)
		{
			this._stringMaps[languageId] = {};
		}

		this._stringMaps[languageId][stringId] = value;
	},
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = LangPack;

//------------------------------------------------------------------------------------------------------------
