//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function SoundEngine()
{
	rpEngine.logger.log('Creating SoundEngine...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
	
	this.setGlobalVolume(rpEngine.config.getVolumeMusic());
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

SoundEngine.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_currentMusic : null,
	_currentMusicId : null,

	//------------------------------------------------------------------------------------------------------------
	//INITIALIZERS - SERIALIZERS
	//------------------------------------------------------------------------------------------------------------
	
	readFromSave : function(data)
	{
		var currentMusicId = data.currentMusicId;
		if (currentMusicId != '')
		{
			this.playMusic(currentMusicId);
		}
		else
		{
			this.stopMusic();
		}
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	playMusic : function(soundId)
	{
		var path = rpEngine.moduleManager.formatCurrentModuleFilePath(rpEngine.assetManager.getPackGame().getSoundPath(soundId));

		rpEngine.logger.log('SoundEngine.playMusic -- Playing sound file "%s" from path "%s".', [soundId, path], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		this.stopMusic();
		
		try
		{
			this._currentMusic = new howler.Howl({
				autoplay : true,
				buffer : true,
				loop : true,
				src : [path],
			});
			this._currentMusicId = soundId;
		}
		catch (exception)
		{
			rpEngine.logger.log('SoundEngine.playMusic -- Error while playing sound file "%s" from path "%s" -- %s', [soundId, path, exception.message], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);	
		}
	},
	
	playSound : function(soundId)
	{
		var path = rpEngine.moduleManager.formatCurrentModuleFilePath(rpEngine.assetManager.getPackGame().getSoundPath(soundId));
		
		try
		{
			new howler.Howl({
				autoplay : true,
				src : [path],
			});
		}
		catch (exception)
		{
			rpEngine.logger.log('SoundEngine.playMusic -- Error while playing sound file "%s" from path "%s" -- %s', [soundId, path, exception.message], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);	
		}
	},
	
	stopMusic : function()
	{
		if (this._currentMusic != null)
		{
			this._currentMusicId = '';
			this._currentMusic.fade(1, 0, 1000);
		}
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------
	
	getCurrentMusicId : function()
	{
		return this._currentMusicId;
	},
	
	setGlobalVolume : function(value)
	{
		Howler.volume(value);
	},
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = SoundEngine;

//------------------------------------------------------------------------------------------------------------