//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function Config()
{
	this.loadFile();
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

Config.FILE_NAME = 'config.json';
Config.FILE_ROLE_VALUE = 'config';

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

Config.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_currentModuleId : '',
	
	_volumeMusic : 1.0,
	_volumeSound : 1.0,
		
	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	createDefaultFile : function()
	{
		rpEngine.logger.log('Creating default configuration...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

		this._currentModuleId = 'immortals';
		
		this._volumeMusic = 1.0;
		this._volumeSound = 1.0;
		
		this.saveFile();
	},
	
	loadFile : function()
	{
		rpEngine.logger.log('Loading configuration...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

		var filePath = appBasePath + 'config/' + rp.Config.FILE_NAME;
		var data = rp.Util.loadFileJson(filePath)
		
		if (data == null)
		{
			rpEngine.logger.log('Config.loadFile -- Could not load config file "%s", creating default.', [filePath], rp.Logger.LEVEL_WARNING, rp.Logger.SUBJECT_ENGINE);
			this.createDefaultFile();
			
			return;
		}

		//Vérification des erreurs
		
		if (data.role != rp.Config.FILE_ROLE_VALUE)
		{
			rpEngine.logger.log('Config.loadFile -- Bad role ID for config file "%s", creating default.', [filePath], rp.Logger.LEVEL_WARNING, rp.Logger.SUBJECT_ENGINE);
			this.createDefaultFile();
		}
		
		//Chargement des données
		
		this._currentModuleId = data.currentModuleId;
		this._volumeMusic = data.volumeMusic;
		this._volumeSound = data.volumeSound;

		rpEngine.logger.log('Configuration loaded successfully.', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
	},
	
	saveFile : function()
	{
		rpEngine.logger.log('Saving configuration...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		var data =
		{
			role : rp.Config.FILE_ROLE_VALUE,
			currentModuleId : this._currentModuleId,
			volumeMusic : this._volumeMusic,
			volumeSound : this._volumeSound,
		};

		rp.Util.createDirectory(appBasePath + 'config');
		var filePath = appBasePath + 'config/' + rp.Config.FILE_NAME;
		rp.Util.saveFileJson(filePath, data);
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------
	
	getVolumeMusic : function()
	{
		return this._volumeMusic;
	},
	setVolumeMusic : function(value)
	{
		this._volumeMusic = value;
	},

	getVolumeSound : function()
	{
		return this._volumeSound;
	},
	setVolumeSound : function(value)
	{
		this._volumeSound = value;
	},
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = Config;

//------------------------------------------------------------------------------------------------------------