//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function Game()
{
	this._actorMap = {};
	this._nodeMap = {};
	this._narrativeActionMap = {};
	this._variableMap = {};

	this._currentNode = null;
	this._currentGameImageId = '';
	this._currentGameTextTokenList = [];
	this._currentInfoTextTokenList = [];
	
	this._tempImageSelectionIdList = [];
	this._tempTextInputStringList = [];
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

Game.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	//Current game data
	
	_actorMap : null, //Actor.id => Actor
	_nodeMap : null, //Node.id => Node
	_narrativeActionMap : null, //Action.id => Action
	_variableMap : null,

	_currentNode : null,
	_currentGameImageId : '',
	_currentGameTextTokenList : null,
	_currentInfoTextTokenList : null,
	
	_currentTimeUnit : '',
	
	_playerActor : null,
	
	//Temp data
	
	_nextNodeIdLocation : '',
	
	_tempNarrativeAction : null,
	_tempImageSelectionIdList : null,
	_tempTextInputStringList : null,

	//------------------------------------------------------------------------------------------------------------
	//INITIALIZERS - SERIALIZERS
	//------------------------------------------------------------------------------------------------------------
	
	initGameLoad : function(data)
	{
		this.initGameVariables();

		this.readFromSave(data);

		this._updateGameState();
		
		return true;
	},
	
	initGameNew : function()
	{
		this.initGameVariables();
		
		//Création des nodes
		/*
		rpEngine.logger.log('Game._createNode -- Creating node map...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_GAME);
		
		var nodeTemplateMap = rpModule.getNodeTemplateMap();
		for (var key in nodeTemplateMap)
		{
			this._createNode(nodeTemplateMap[key])
		}
		*/
		
		//Initialisation du premier node de jeu

		rpEngine.logger.log('Game._createNode -- Initializing first game node...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_GAME);
		
		var nodeIdNewGame = rpModule.getDefaultData(rp.Module.DEFAULT_DATA_NODE_ID_NEW_GAME);
		this._nextNodeIdLocation = nodeIdNewGame;
		
		var result = this._enterNextNode();
		if (result == false)
		{
			rpEngine.logger.log('Game.initGameNew -- Error while initializing first game node "%s".', [nodeIdNewGame], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return false;
		}
		
		return true;
	},
	
	initGameVariables : function()
	{
		this.clearBackgroundImage();
		this.clearNarrativeActionMap();
		this.clearTextAll();
		
		this._narrativeActionMap = {};
		this._actorMap = {};
		this._nodeMap = {};
		this._variableMap = {};
		
		this._playerActor = null;
		
		this._tempGameAction = null;
		this._tempImageSelectionIdList = [];
		this._tempTextInputStringList = [];
	},
	
	readFromSave : function(data)
	{
		rpEngine.logger.log('Game.readFromSave -- Deserializing actor map...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_GAME);
		
		for (var id in data.actorSaveMap)
		{
			var actor = rp.Actor.createFromSave(data.actorSaveMap[id]);
			this._actorMap[id] = actor;
		}

		rpEngine.logger.log('Game.readFromSave -- Deserializing node map...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_GAME);
		
		for (var id in data.nodeSaveMap)
		{
			var nodeTemplate = rpModule.getNodeTemplate(id);
			
			if (nodeTemplate == null)
			{
				rpEngine.logger.log('Game.readFromSave -- Error while reading node "%s", node template "%s" does not exist.', [data.id, nodeTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
				continue;
			}
			
			this.getNode(id).readFromSave(data.nodeSaveMap[id]);
		}
		rpEngine.logger.log('Game.readFromSave -- Deserializing narrative action map...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_GAME);
		
		for (var id in data.narrativeActionSaveMap)
		{
			var narrativeAction = rp.NarrativeAction.createFromSave(data.narrativeActionSaveMap[id]);
			this._narrativeActionMap[id] = narrativeAction;
		}

		rpEngine.logger.log('Game.readFromSave -- Deserializing variable map...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_GAME);

		this._variableMap = data.variableMap;
		
		rpEngine.logger.log('Game.readFromSave -- Deserializing text token lists...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_GAME);
		
		for (var key in data.currentGameTextTokenSaveList)
		{
			var textTokenSave = data.currentGameTextTokenSaveList[key];
			if (textTokenSave.tokenType == 'lang')
			{
				var textTokenLang = rp.TextTokenLang.createFromSave(textTokenSave);
				this._currentGameTextTokenList.push(textTokenLang);
			}
			else if (textTokenSave.tokenType == 'raw')
			{
				var textTokenRaw = rp.TextTokenRaw.createFromSave(textTokenSave);
				this._currentGameTextTokenList.push(textTokenRaw);
			}
			else
			{
				rpEngine.logger.log('GameSave.readFromSave -- Error, unknown text token type "%s".', [textTokenSave.tokenType], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
			}
		}

		for (var key in data.currentInfoTextTokenSaveList)
		{
			var textTokenSave = data.currentInfoTextTokenSaveList[key];
			if (textTokenSave.tokenType == 'lang')
			{
				var textTokenLang = rp.TextTokenLang.createFromSave(textTokenSave);
				this._currentInfoTextTokenList.push(textTokenLang);
			}
			else if (textTokenSave.tokenType == 'raw')
			{
				var textTokenRaw = rp.TextTokenRaw.createFromSave(textTokenSave);
				this._currentInfoTextTokenList.push(textTokenRaw);
			}
			else
			{
				rpEngine.logger.log('GameSave.readFromSave -- Error, unknown text token type "%s".', [textTokenSave.tokenType], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
			}
		}
		
		rpEngine.logger.log('Game.readFromSave -- Deserializing general game data...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_GAME);
		
		this._currentNode = this.getNode(data.currentNodeId);
		this._currentGameImageId = data.currentGameImageId;
		this.setPlayerActor(this.getActor(data.playerActorId));

		rpEngine.logger.log('Game.readFromSave -- Deserializing sound engine data...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_GAME);

		rpEngine.soundEngine.readFromSave(data.soundEngineSave);

		rpEngine.logger.log('Game.readFromSave -- Finished deserializing save data.', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	_checkHasNextNode : function()
	{
		return (this._nextNodeIdLocation != '');
	},
	
	_createNode : function(nodeTemplate)
	{
		var nodeClass = rpEngine.assetManager.getPackGame().getScriptData(nodeTemplate.getScriptId());
		if (nodeClass == null)
		{
			rpEngine.logger.log('Game._createNode -- Error while creating node, cannot find script "%s".', [nodeTemplate.getScriptId()], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		var node = new nodeClass();
		if (node == null)
		{
			rpEngine.logger.log('Game._createNode -- Error while creating node, could not create new script "%s".', [nodeTemplate.getScriptId()], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		node.initFromTemplate(nodeTemplate);
		
		this._nodeMap[nodeTemplate.getId()] = node;
	},

	_enterNextNode : function()
	{
		var nodeId = this._nextNodeIdLocation;
		this._nextNodeIdLocation = '';
		
		if (nodeId == '')
		{
			rpEngine.logger.log('Game._enterNextNode -- Error, no node set to enter.', [], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return false;
		}
		
		var node = this.getNode(nodeId);

		if (node == null)
		{
			rpEngine.logger.log('Game._enterNextNode -- Error, cannot find node "%s".', [nodeId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return false;
		}
		
		rpEngine.logger.log('Game._enterNextNode -- Entering node "%s".', [nodeId], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_GAME);
		
		if (node.enterNode == null)
		{
			rpEngine.logger.log('Game._enterNextNode -- Error, no "enterNode" function found for node "%s".', [nodeId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return false;
		}
		
		this._currentNode = node;
		node.enterNode();
		
		this._updateGameState();
		
		return true;
	},
	
	_updateGameState : function()
	{
		while (this._checkHasNextNode() == true)
		{
			this._enterNextNode();
		}
		
		rpEngine.gameScreen.getMainPanel().updateGameImage();
		rpEngine.gameScreen.getMainPanel().updateGameText();
		rpEngine.gameScreen.getMainPanel().updateNarrativeActionList();
	},
	
	addActor : function(actorId)
	{
		if (this._actorMap[actorId] != null)
		{
			rpEngine.logger.log('Game.addActor -- Actor with ID "%s" already exists.', [actorId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return null;
		}
		
		var actor = rp.Actor.create(actorId);
		this._actorMap[actorId] = actor;
		
		return actor;
	},
	
	addNarrativeAction : function(actionId, functionName, langTitleId, langDescriptionId, langInfoId, options)
	{
		if (actionId == '')
		{
			var generate = true;
			while (generate == true)
			{
				actionId = rp.Util.generateUuid();
				generate = (this._narrativeActionMap[actionId] != null);
			}
		}
		
		if (this._narrativeActionMap[actionId] != null)
		{
			rpEngine.logger.log('Game.addNarrativeAction -- Narrative action with ID "%s" already exists.', [actorId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return null;
		}
		
		var narrativeAction = rp.NarrativeAction.create(actionId, functionName, langTitleId, langDescriptionId, langInfoId, options);
		this._narrativeActionMap[actionId] = narrativeAction;
		
		return narrativeAction;
	},

	addTextGameLang : function(stringId, replaceValueList)
	{
		this._currentGameTextTokenList.push(rp.TextTokenLang.create(stringId, replaceValueList));
	},
	
	addTextGameRaw : function(string, replaceValueList)
	{
		this._currentGameTextTokenList.push(rp.TextTokenRaw.create(string, replaceValueList));	
	},
	
	addTextInfoLang : function(stringId, replaceValueList)
	{
		this._currentInfoTextTokenList.push(rp.TextTokenLang.create(stringId, replaceValueList));
	},
	
	addTextInfoRaw : function(string, replaceValueList)
	{
		this._currentInfoTextTokenList.push(rp.TextTokenRaw.create(string, replaceValueList));	
	},
	
	clearBackgroundImage : function()
	{
		this._currentGameImageId = '';
	},
	
	clearTextAll : function()
	{
		this.clearTextGame();
		this.clearTextInfo();
	},

	clearTextGame : function()
	{
		//Ne pas mettre "...length = 0" pour permettre de préserver temporairement la référence en cas de bug
		this._currentGameTextTokenList = [];
	},
	
	clearTextInfo : function()
	{
		//Ne pas mettre "...length = 0" pour permettre de préserver temporairement la référence en cas de bug
		this._currentInfoTextTokenList = [];	
	},
	
	clearNarrativeActionMap : function()
	{
		//Ne pas mettre "...length = 0" pour permettre de préserver temporairement la référence en cas de bug
		this._narrativeActionMap = {};
	},
	
	executeNarrativeAction : function(narrativeActionId)
	{
		var narrativeAction = this._narrativeActionMap[narrativeActionId];
		if (narrativeAction == null)
		{
			rpEngine.logger.log('Game.executeNarrativeAction -- Error, action "%s" does not exist.', [narrativeActionId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
		}
		
		var functionName = narrativeAction.getFunctionName();

		rpEngine.logger.log(
				'Game.executeNarrativeAction -- Executing action, function "%s", ID "%s", node "%s".',
				[functionName, narrativeActionId, this._currentNode.getId()], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		if (this._currentNode[functionName] == null)
		{
			rpEngine.logger.log(
					'Game.executeNarrativeAction -- Error executing action "%s", current node "%s" has no function "%s".',
					[narrativeActionId, this._currentNode.getId(), functionName], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		var functionData = {};
		functionData.challengeRollList = [];
		functionData.imageSelectionList = [];
		functionData.textInputList = [];
		
		//Text input
		
		if (this._tempTextInputStringList.length < narrativeAction.getNarrativeActionTextInputDataList().length)
		{
			rpEngine.logger.log('Game.executeNarrativeAction -- Showing text input index "%s".', [this._tempTextInputStringList.length], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_GAME);
			
			this._tempNarrativeAction = narrativeAction;

			var dataList = narrativeAction.getNarrativeActionTextInputDataList();
			var narrativeActionTextInputData = dataList[this._tempTextInputStringList.length];
			
			rpEngine.gameScreen.getPopupTextInput().show(narrativeActionTextInputData);
			return;
		}
		else if (narrativeAction.getNarrativeActionTextInputDataList().length > 0)
		{
			for (var i = 0; i < this._tempTextInputStringList.length; i++)
			{
				functionData.textInputList.push(this._tempTextInputStringList);
			}
		}

		//Image selection
		
		if (this._tempImageSelectionIdList.length < narrativeAction.getNarrativeActionImageSelectionDataList().length)
		{
			rpEngine.logger.log('Game.executeNarrativeAction -- Showing image selection index "%s".', [this._tempImageSelectionIdList.length], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_GAME);
			
			this._tempNarrativeAction = narrativeAction;

			var dataList = narrativeAction.getNarrativeActionImageSelectionDataList();
			var narrativeActionImageSelectionData = dataList[this._tempImageSelectionIdList.length];
			
			rpEngine.gameScreen.getPopupImageSelection().show(narrativeActionImageSelectionData);
			return;
		}
		else if (narrativeAction.getNarrativeActionImageSelectionDataList().length > 0)
		{
			for (var i = 0; i < this._tempImageSelectionIdList.length; i++)
			{
				functionData.imageSelectionList.push(this._tempImageSelectionIdList);
			}
		}
		
		//Options
		
		var narrativeActionOptions = narrativeAction.getOptions();
		var clearText = ((narrativeActionOptions[rp.NarrativeAction.OPTION_CLEAR_TEXT] != null) ? narrativeActionOptions[rp.NarrativeAction.OPTION_CLEAR_TEXT] : true);
		var clearNarrativeActionMap = 
				((narrativeActionOptions[rp.NarrativeAction.OPTION_CLEAR_NARRATIVE_ACTION_MAP] != null) ? narrativeActionOptions[rp.NarrativeAction.OPTION_CLEAR_NARRATIVE_ACTION_MAP] : true);
		
		//Call action
		
		//TODO: Message de debug pour le contenu de functionData?
		
		var tempGameTextTokenList = this._currentGameTextTokenList;
		var tempInfoTextTokenList = this._currentInfoTextTokenList;
		var tempNarrativeActionMap = this._narrativeActionMap;

		if (clearText == true)
		{
			this.clearTextAll();
		}
		
		if (clearNarrativeActionMap == true)
		{
			this.clearNarrativeActionMap();
		}

		var functionToCall = this._currentNode[functionName];
		try
		{
			functionToCall(functionData);
		}
		catch (exception)
		{
			rpEngine.logger.log('Game.executeNarrativeAction -- Exception while executing narrative action: \n%s.', [exception.stack], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);

			this._currentGameTextTokenList = tempGameTextTokenList;
			this._currentInfoTextTokenList = tempInfoTextTokenList;
			this._narrativeActionMap = tempNarrativeActionMap;
		}
		
		this._updateGameState();
	},

	executeNarrativeActionInputCancel : function()
	{
		this._tempGameAction = null;
		this._tempImageSelectionIdList.length = 0;
		this._tempTextInputStringList.length = 0;
	},

	executeNarrativeActionInputImageSelectionConfirm : function(imageId)
	{
		var narrativeAction = this._tempNarrativeAction;
		
		this._tempNarrativeAction = null;
		this._tempImageSelectionIdList.push(imageId);

		this.executeNarrativeAction(narrativeAction.getId());
	},
	
	executeNarrativeActionInputTextInputConfirm : function(inputString)
	{
		var narrativeAction = this._tempNarrativeAction;
		
		this._tempNarrativeAction = null;
		this._tempTextInputStringList.push(inputString);

		this.executeNarrativeAction(narrativeAction.getId());
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------
	
	getActor : function(id)
	{
		return this._actorMap[id];
	},
	getActorMap : function()
	{
		return this._actorMap;
	},
	
	getCurrentGameImageId : function()
	{
		return this._currentGameImageId;
	},
	setCurrentGameImageId : function(value)
	{
		this._currentGameImageId = value;
	},
	
	getCurrentGameTextTokenList : function()
	{
		return this._currentGameTextTokenList;
	},

	getCurrentInfoTextTokenList : function()
	{
		return this._currentInfoTextTokenList;
	},
	
	getCurrentNode : function()
	{
		return this._currentNode;
	},
	
	getNarrativeActionMap : function()
	{
		return this._narrativeActionMap;
	},
	
	getNextNodeIdLocation : function()
	{
		return this._nextNodeIdLocation;
	},
	setNextNodeIdLocation : function(value)
	{
		this._nextNodeIdLocation = value;
	},
	
	getNode : function(id)
	{
		if (this._nodeMap[id] == null)
		{
			var nodeTemplate = rpModule.getNodeTemplate(id);
			if (nodeTemplate == null)
			{
				rpEngine.logger.log('Game.getNode -- Error creating node, node template "%s" does not exist.', [id], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
				return null;
			}
			
			this._createNode(nodeTemplate);
		}
		
		return this._nodeMap[id];
	},
	
	getNodeMap : function()
	{
		return this._nodeMap;
	},
	
	getPlayerActor : function()
	{
		return this._playerActor;
	},
	setPlayerActor : function(value)
	{
		this._playerActor = value;
		
		rpEngine.gameScreen.getPlayerCharacterPanel().setPlayerActor(this._playerActor);
		rpEngine.gameScreen.getInventoryPanel().getInventoryPanelSub1().setInventory((this._playerActor != null) ? this._playerActor.getInventory() : null);
	},

	getVariable : function(variableId)
	{
		return this._variableMap[variableId];
	},
	getVariableMap : function()
	{
		return this._variableMap;
	},
	setVariable : function(variableId, value)
	{
		this._variableMap[variableId] = value;
	},
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = Game;

//------------------------------------------------------------------------------------------------------------