//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function NarrativeAction()
{
	this._narrativeActionImageSelectionDataList = [];
	this._narrativeActionTextInputDataList = [];
	
	this._options = {};
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

NarrativeAction.OPTION_CLEAR_NARRATIVE_ACTION_MAP = 'clearNarrativeActionMap';
NarrativeAction.OPTION_CLEAR_TEXT = 'clearText';

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

NarrativeAction.create = function(id, functionName, langTitleId, langDescriptionId, langInfoId, options)
{
	var narrativeAction = new NarrativeAction();
	
	narrativeAction._id = id;

	narrativeAction._langDescriptionId = langDescriptionId;
	narrativeAction._langInfoId = langInfoId;
	narrativeAction._langTitleId = langTitleId;
	
	narrativeAction._functionName = functionName;
	
	narrativeAction._options = (options != null ? options : {});
	
	return narrativeAction;
}

NarrativeAction.createFromSave = function(data)
{
	var narrativeAction = new NarrativeAction();
	
	narrativeAction._id = data.id;
	
	narrativeAction._langDescriptionId = data.langDescriptionId;
	narrativeAction._langInfoId = data.langInfoId;
	narrativeAction._langTitleId = data.langTitleId;
	
	narrativeAction._functionName = data.functionName;
	
	narrativeAction._options = data.options;

	for (var key in data.narrativeActionImageSelectionDataSaveList)
	{
		var narrativeActionImageSelectionData = rp.NarrativeActionImageSelectionData.createFromSave(data.narrativeActionImageSelectionDataSaveList[key]);
		narrativeAction._narrativeActionImageSelectionDataList.push(narrativeActionImageSelectionData);
	}

	for (var key in data.narrativeActionTextInputDataSaveList)
	{
		var narrativeActionTextInputData = rp.NarrativeActionTextInputData.createFromSave(data.narrativeActionTextInputDataSaveList[key]);
		narrativeAction._narrativeActionTextInputDataList.push(narrativeActionTextInputData);
	}
	
	return narrativeAction;
}

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

NarrativeAction.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_id : '',
	
	_langDescriptionId : '',
	_langInfoId : '',
	_langTitleId : '',
	
	_functionName : '',

	_narrativeActionImageSelectionDataList : null,
	_narrativeActionTextInputDataList : null,
	
	_options : null,
	
	//------------------------------------------------------------------------------------------------------------
	//INITIALIZERS - SERIALIZERS
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	addNarrativeActionImageSelectionData : function(langMessageId, imageIdList)
	{
		if (imageIdList.length == 0)
		{
			rpEngine.logger.log('NarrativeAction.addNarrativeActionImageSelectionData -- Image ID list is empty.', [], rp.Logger.LEVEL_WARNING, rp.Logger.SUBJECT_GAME);
			return;
		}

		this._narrativeActionImageSelectionDataList.push(rp.NarrativeActionImageSelectionData.create(langMessageId, imageIdList));
	},
	
	addNarrativeActionTextInputData : function(langMessageId, inputLengthMin, inputLengthMax)
	{
		if ((inputLengthMin > inputLengthMax) && (inputLengthMax > 0))
		{
			rpEngine.logger.log(
					'NarrativeAction.addNarrativeActionTextInputData -- Min length "%s" is greater than max length "%s".',
					[inputLengthMin, inputLengthMax], rp.Logger.LEVEL_WARNING, rp.Logger.SUBJECT_GAME);
		}
		
		this._narrativeActionTextInputDataList.push(rp.NarrativeActionTextInputData.create(langMessageId, inputLengthMin, inputLengthMax));
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------

	getFunctionName : function()
	{
		return this._functionName;
	},
	setFunctionName : function(value)
	{
		this._functionName = value;
	},

	getId : function()
	{
		return this._id;
	},
	setId : function(value)
	{
		this._id = value;
	},

	getLangDescriptionId : function()
	{
		return this._langDescriptionId;
	},
	setLangDescriptionId : function(value)
	{
		this._langDescriptionId = value;
	},

	getLangInfoId : function()
	{
		return this._langInfoId;
	},
	setLangInfoId : function(value)
	{
		this._langInfoId = value;
	},

	getLangTitleId : function()
	{
		return this._langTitleId;
	},
	setLangTitleId : function(value)
	{
		this._langTitleId = value;
	},
	
	getNarrativeActionImageSelectionDataList : function()
	{
		return this._narrativeActionImageSelectionDataList;
	},

	getNarrativeActionTextInputDataList : function()
	{
		return this._narrativeActionTextInputDataList;
	},

	getOptions : function()
	{
		return this._options;
	},
	setOptions : function(value)
	{
		this._options = value;
	},

	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = NarrativeAction;

//------------------------------------------------------------------------------------------------------------