//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function ActorEffect(tempRawData)
{
	this._tempRawData = tempRawData;
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

ActorEffect.TYPE_CHANGE_VALUE_CURRENT = 'changeValueCurrent';
ActorEffect.TYPE_CHANGE_VALUE_MAX = 'changeValueMax';
ActorEffect.TYPE_CHANGE_VALUE_MIN = 'changeValueMin';

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

ActorEffect.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_parentActorStatTemplate : null,
	_parentInventoryItemTemplate : null,
		
	_effectType : null,
	_targetActorStatTemplate : null,
	_value : null,
	_factor : null,
	
	_tempRawData : null,

	//------------------------------------------------------------------------------------------------------------
	//INITIALIZERS - SERIALIZERS
	//------------------------------------------------------------------------------------------------------------

	initFromTempRawData : function()
	{	
		var initResult = true;
		
		var data = this._tempRawData;

		this._effectType = data.type;
		this._factor = ((data.factor != null) ? parseInt(data.factor) : null);
		this._value = ((data.value != null) ? parseInt(data.value) : null);

		var targetActorStatTemplateId = data.statId;
		this._targetActorStatTemplate = rpModule.getActorStatTemplate(targetActorStatTemplateId);
		
		if ((this._effectType != ActorEffect.TYPE_CHANGE_VALUE_CURRENT) &&
			(this._effectType != ActorEffect.TYPE_CHANGE_VALUE_MAX) &&
			(this._effectType != ActorEffect.TYPE_CHANGE_VALUE_MIN))
		{
			rpEngine.logger.log('ActorEffect.initFromTempRawData -- Type "%s" does not exist.', [this._effectType], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			initResult = false;
		}

		if (this._targetActorStatTemplate == null)
		{
			rpEngine.logger.log('ActorEffect.initFromTempRawData -- Actor stat template "%s" does not exist.', [targetActorStatTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			initResult = false;
		}
		
		if ((this._value == null) && (this._factor == null))
		{
			rpEngine.logger.log('ActorEffect.initFromTempRawData -- Effect has no value or factor.', [], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			initResult = false;
		}
		
		return initResult;
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------
	
	getEffectType : function()
	{
		return this._effectType;
	},
	
	getFactor : function()
	{
		return this._factor;
	},
	
	getParentActorStatTemplate : function()
	{
		return this._parentActorStatTemplate;
	},
	setParentActorStatTemplate : function(value)
	{
		this._parentActorStatTemplate = value;
	},

	getParentInventoryItemTemplate : function()
	{
		return this._parentInventoryItemTemplate;
	},
	setParentInventoryItemTemplate : function(value)
	{
		this._parentInventoryItemTemplate = value;
	},
	
	getTargetActorStatTemplate : function()
	{
		return this._targetActorStatTemplate;
	},
	
	getValue : function()
	{
		return this._value;
	},
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = ActorEffect;

//------------------------------------------------------------------------------------------------------------