//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function ActorStat()
{
	this._sourceActorEffectList = [];
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

ActorStat.create = function(parentActor, actorStatTemplate)
{
	var actorStat = new ActorStat();

	actorStat._parentActor = parentActor;
	actorStat._actorStatTemplate = actorStatTemplate;
	
	actorStat._valueCurrentBase = actorStatTemplate.getValueDefault();
	actorStat._valueMaxBase = actorStatTemplate.getValueDefaultMax();
	actorStat._valueMinBase = actorStatTemplate.getValueDefaultMin();
	actorStat._valueProgressCurrent = 0;
	
	actorStat.updateAllValuesFinal();
	
	return actorStat;
}

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

ActorStat.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_parentActor : null,
	_actorStatTemplate : null,
	
	_valueCurrentBase : 0,
	_valueCurrentEffects : 0,
	_valueCurrentFinal : 0,
	_valueMaxBase : 0,
	_valueMaxEffects : 0,
	_valueMaxFinal : 0,
	_valueMinBase : 0,
	_valueMinEffects : 0,
	_valueMinFinal : 0,
	_valueProgressCurrent : 0,
	
	_sourceActorEffectList : null, //Effects which modify this stat

	//------------------------------------------------------------------------------------------------------------
	//INITIALIZERS - SERIALIZERS
	//------------------------------------------------------------------------------------------------------------

	readFromSave : function(data)
	{
		this.setValueCurrentBase(data.valueCurrentBase);
		this.setValueMaxBase(data.valueMaxBase);
		this.setValueMinBase(data.valueMinBase);
		this.setValueProgressCurrent(data.valueProgressCurrent);
	},

	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	_calculateActorEffectValue : function(actorEffect)
	{
		if (actorEffect.getParentActorStatTemplate() != null)
		{
			var sourceActorStat = this._parentActor.getStat(actorEffect.getParentActorStatTemplate().getId());
			if (sourceActorStat.getValueCurrentBase() > 0)
			{
				if (actorEffect.getValue() != null)
				{
					return actorEffect.getValue();
				}
				else if (actorEffect.getFactor() != null)
				{
					return sourceActorStat.getValueCurrentBase() * actorEffect.getFactor();
				}
			}
		}
		else if (actorEffect.getParentInventoryItemTemplate() != null)
		{
			var sourceInventoryItem = this._parentActor.getInventory().getItem(actorEffect.getParentInventoryItemTemplate().getId());
			if (sourceInventoryItem != null)
			{
				if (sourceInventoryItem.getItemCountEquipped() > 0)
				{
					if (actorEffect.getValue() != null)
					{
						return actorEffect.getValue();
					}
					else if (actorEffect.getFactor() != null)
					{
						return sourceInventoryItem.getItemCountEquipped() * actorEffect.getFactor();
					}
				}
			}
		}
		
		return 0;
	},
	
	_updateValueCurrentFinal : function()
	{
		this._valueCurrentEffects = 0;
		for (var i = 0; i < this._sourceActorEffectList.length; i++)
		{
			var actorEffect = this._sourceActorEffectList[i];
			if (actorEffect.getEffectType() == rp.ActorEffect.TYPE_CHANGE_VALUE_CURRENT)
			{
				this._valueCurrentEffects += this._calculateActorEffectValue(actorEffect);
			}
		}
		
		this._valueCurrentFinal = this._valueCurrentBase + this._valueCurrentEffects;
		
		if ((this._valueMaxFinal !== null) && (this._valueCurrentFinal > this._valueMaxFinal))
		{
			this._valueCurrentFinal = this._valueMaxFinal;
		}
		else if ((this._valueMinFinal !== null) && (this._valueCurrentFinal < this._valueMinFinal))
		{
			this._valueCurrentFinal = this._valueMinFinal;
		}
		
		this._parentActor.updateRegisteredPanelListStat(this);
	},

	_updateValueMaxFinal : function()
	{
		if (this._valueMaxBase != null)
		{
			this._valueMaxEffects = 0;
			for (var i = 0; i < this._sourceActorEffectList.length; i++)
			{
				var actorEffect = this._sourceActorEffectList[i];
				if (actorEffect.getEffectType() == rp.ActorEffect.TYPE_CHANGE_VALUE_MAX)
				{
					this._valueMaxEffects += this._calculateActorEffectValue(actorEffect);
				}
			}

			this._valueMaxFinal = this._valueMaxBase + this._valueMaxEffects;
		}
		else
		{
			this._valueMaxEffects = null;
			this._valueMaxFinal = null;
		}
		
		this._updateValueCurrentFinal();
	},

	_updateValueMinFinal : function()
	{
		if (this._valueMinBase != null)
		{
			this._valueMinEffects = 0;
			for (var i = 0; i < this._sourceActorEffectList.length; i++)
			{
				var actorEffect = this._sourceActorEffectList[i];
				if (actorEffect.getEffectType() == rp.ActorEffect.TYPE_CHANGE_VALUE_MIN)
				{
					this._valueMinEffects += this._calculateActorEffectValue(actorEffect);
				}
			}

			this._valueMinFinal = this._valueMinBase + this._valueMinEffects;
		}
		else
		{
			this._valueMinEffects = null;
			this._valueMinFinal = null;
		}
		
		this._updateValueCurrentFinal();
	},
	
	updateAllValuesFinal : function()
	{
		this._updateValueCurrentFinal();
		this._updateValueMaxFinal();
		this._updateValueMinFinal();
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------

	getActorStatCategory : function()
	{
		return this._actorStatTemplate.getActorStatCategory();
	},
	
	getActorStatTemplate : function()
	{
		return this._actorStatTemplate;
	},

	getId : function()
	{
		return this._actorStatTemplate.getId();
	},
	
	getParentActor : function()
	{
		return this._parentActor;
	},

	getSourceActorEffectList : function()
	{
		return this._sourceActorEffectList;
	},
	
	getValueCurrentBase : function()
	{
		return this._valueCurrentBase;
	},
	setValueCurrentBase : function(value)
	{
		this._valueCurrentBase = value;
		this._updateValueCurrentFinal();
		
		this._parentActor.updateActorStatTargetActorEffectsFromActorStat(this);
	},
	
	getValueCurrentEffects : function()
	{
		return this._valueCurrentEffects;
	},
	
	getValueCurrentFinal : function()
	{
		return this._valueCurrentFinal;
	},
	
	getValueMaxBase : function()
	{
		return this._valueMaxBase;
	},
	setValueMaxBase : function(value)
	{
		if (value != null)
		{
			this._valueMaxBase = value;
		}
		else
		{
			this._valueMaxBase = null;
		}
		
		this._updateValueMaxFinal();
	},
	
	getValueMaxEffects : function()
	{
		return this._valueMaxEffects;
	},
	
	getValueMaxFinal : function()
	{
		return this._valueMaxFinal;
	},
	
	getValueMinBase : function()
	{
		return this._valueMinBase;
	},
	setValueMinBase : function(value)
	{
		if (value != null)
		{
			this._valueMinBase = value;
		}
		else
		{
			this._valueMinBase = null;
		}
		
		this._updateValueMinFinal();
	},
	
	getValueMinEffects : function()
	{
		return this._valueMinEffects;
	},
		
	getValueMinFinal : function()
	{
		return this._valueMinFinal;
	},
	
	getValueProgressCurrent : function()
	{
		return this._valueProgressCurrent;
	},
	setValueProgressCurrent : function(value)
	{
		this._valueProgressCurrent = value;
	},
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = ActorStat;

//------------------------------------------------------------------------------------------------------------