//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function ActorStatTemplate(id)
{
	this._actorEffectList = [];
	this._id = id;
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

ActorStatTemplate.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_id : '',

	_actorEffectList : null,
	_actorStatCategory : null,
	
	_langDescriptionId : '',
	_langInfoId : '',
	_langNameId : '',
	
	_order : 0,
	
	_valueDefault : 0.0,
	_valueDefaultMax : 0.0,
	_valueDefaultMin : 0.0,

	//------------------------------------------------------------------------------------------------------------
	//INITIALIZERS - SERIALIZERS
	//------------------------------------------------------------------------------------------------------------

	initFromFile : function(data)
	{
		var initResult = true;

		this._langDescriptionId = ((data.langDescriptionId != null) ? data.langDescriptionId : '');
		this._langInfoId = ((data.langInfoId != null) ? data.langInfoId : '');
		this._langNameId = ((data.langNameId != null) ? data.langNameId : '');
		this._order = ((data.order != null) ? data.order : 1);
		this._valueDefault = ((data.valueDefault != null) ? parseInt(data.valueDefault) : 0);
		this._valueDefaultMax = ((data.valueDefaultMax != null) ? parseInt(data.valueDefaultMax) : null);
		this._valueDefaultMin = ((data.valueDefaultMin != null) ? parseInt(data._valueDefaultMin) : null);

		var actorEffectDataList = ((data.effectList != null) ? data.effectList : []);
		for (var i = 0; i < actorEffectDataList.length; i++)
		{
			var actorEffect = new rp.ActorEffect(actorEffectDataList[i]);
			actorEffect.setParentActorStatTemplate(this);
			
			this._actorEffectList.push(actorEffect);
		}
		
		var actorStatCategoryId = data.categoryId;
		this._actorStatCategory = rpModule.getActorStatCategory(actorStatCategoryId);
		if (this._actorStatCategory == null)
		{
			rpEngine.logger.log(
					'ActorStatTemplate.initFromFile -- Error, category "%s" does not exist for actor stat template "%s".',
					[this._id, actorStatCategoryId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			initResult = false;
		}
		
		return initResult;
	},
	
	initActorEffectList : function()
	{
		for (var i = this._actorEffectList.length - 1; i >= 0; i--)
		{
			var actorEffect = this._actorEffectList[i]; 
			
			var result = actorEffect.initFromTempRawData();
			if (actorEffect.getTargetActorStatTemplate() === this)
			{
				rpEngine.logger.log('ActorStatTemplate.initActorEffectList -- Actor effect cannot target parent actor stat template.', [], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
				result = false;
			}
			
			if (result == false)
			{
				rpEngine.logger.log('ActorStatTemplate.initActorEffectList -- Error with actor effect "%s" in actor stat template "%s".', [i, this._id], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
				this._actorEffectList.splice(i, 1);
			}
		}
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------
	
	getActorStatCategory : function()
	{
		return this._actorStatCategory;
	},

	getActorEffectList : function()
	{
		return this._actorEffectList;
	},
	
	getId : function()
	{
		return this._id;
	},

	getLangDescription : function()
	{
		return ((this._langDescriptionId != '') ? rpEngine.langManager.getPackGame().getString(this._langDescriptionId) : '');
	},

	getLangDescriptionId : function()
	{
		return this._langDescriptionId;
	},

	getLangInfo : function()
	{
		return ((this._langInfoId != '') ? rpEngine.langManager.getPackGame().getString(this._langInfoId) : '');
	},

	getLangInfoId : function()
	{
		return this._langInfoId;
	},

	getLangName : function()
	{
		return ((this._langNameId != '') ? rpEngine.langManager.getPackGame().getString(this._langNameId) : '');
	},

	getLangNameId : function()
	{
		return this._langNameId;
	},

	getOrder : function()
	{
		return this._order;
	},
	
	getValueDefault : function()
	{
		return this._valueDefault;
	},

	getValueDefaultMax : function()
	{
		return this._valueDefaultMax;
	},

	getValueDefaultMin : function()
	{
		return this._valueDefaultMin;
	},
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = ActorStatTemplate;

//------------------------------------------------------------------------------------------------------------