//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function InventoryItemTemplate(id)
{
	this._id = id;
	
	this._equipActorEffectList = [];
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

InventoryItemTemplate.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_id : '',
	
	_imageId : '',
	_langDescriptionId : '',
	_langInfoId : '',
	_langNameId : '',
	
	_isEquippable : false,
	_isUseable : false,
	_inventoryItemCategory : null,
	
	_equipActorEffectList : null,
	
	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	initFromFile : function(data)
	{
		var initResult = true;

		this._imageId = ((data.isEquippable != null) ? data.imageId : '');
		this._isEquippable = ((data.isEquippable != null) ? data.isEquippable : '');
		this._isUseable = ((data._isUseable != null) ? data._isUseable : '');
		this._langDescriptionId = ((data.langDescriptionId != null) ? data.langDescriptionId : '');
		this._langInfoId = ((data.langInfoId != null) ? data.langInfoId : '');
		this._langNameId = ((data.langNameId != null) ? data.langNameId : '');

		var actorEffectDataList = ((data.equipEffectList != null) ? data.equipEffectList : []);
		for (var i = 0; i < actorEffectDataList.length; i++)
		{
			var actorEffect = new rp.ActorEffect(actorEffectDataList[i]);
			actorEffect.setParentInventoryItemTemplate(this);
			
			this._equipActorEffectList.push(actorEffect);
		}
		
		var inventoryItemCategoryId = data.categoryId;
		this._inventoryItemCategory = rpModule.getInventoryItemCategory(inventoryItemCategoryId);
		if (this._inventoryItemCategory == null)
		{
			rpEngine.logger.log(
					'InventoryItemTemplate.initFromFile -- Error, category "%s" does not exist for inventory item template "%s".',
					[inventoryItemCategoryId, this._id], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			initResult = false;
		}
		
		return initResult;
	},

	initActorEffectList : function()
	{
		for (var i = this._equipActorEffectList.length - 1; i >= 0; i--)
		{
			var actorEffect = this._equipActorEffectList[i]; 
			
			var result = actorEffect.initFromTempRawData();
			if (actorEffect.getTargetActorStatTemplate() === this)
			{
				rpEngine.logger.log('InventoryItemTemplate.initActorEffectList -- Actor effect cannot target parent inventory item template.', [], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
				result = false;
			}
			
			if (result == false)
			{
				rpEngine.logger.log(
						'InventoryItemTemplate.initActorEffectList -- Error with actor effect "%s" in inventory item template "%s".', [i, this._id], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
				this._equipActorEffectList.splice(i, 1);
			}
		}
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------

	getEquipActorEffectList : function()
	{
		return this._equipActorEffectList;
	},

	getId : function()
	{
		return this._id;
	},
	
	getInventoryItemCategory : function()
	{
		return this._inventoryItemCategory;
	},

	getImageId : function()
	{
		return this._imageId;
	},

	getIsEquippable : function()
	{
		return this._isEquippable;
	},

	getIsUsable : function()
	{
		return this._isUsable;
	},
	
	getLangDescription : function()
	{
		return ((this._langDescriptionId != '') ? rpEngine.langManager.getPackGame().getString(this._langDescriptionId) : '');
	},

	getLangDescriptionId : function()
	{
		return this._langDescriptionId;
	},

	getLangInfo : function()
	{
		return ((this._langInfoId != '') ? rpEngine.langManager.getPackGame().getString(this._langInfoId) : '');
	},

	getLangInfoId : function()
	{
		return this._langInfoId;
	},

	getLangName : function()
	{
		return ((this._langNameId != '') ? rpEngine.langManager.getPackGame().getString(this._langNameId) : '');
	},

	getLangNameId : function()
	{
		return this._langNameId;
	},
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = InventoryItemTemplate;

//------------------------------------------------------------------------------------------------------------