//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function GameSave(game)
{
	this.actorSaveMap = {};
	this.nodeSaveMap = {};
	this.narrativeActionSaveMap = {};
	this.variableMap = {};

	this.currentGameTextTokenSaveList = [];
	this.currentInfoTextTokenSaveList = [];

	//Write data

	this.currentModuleId = '';
	this.version = '';

	rpEngine.logger.log('RpEngine.newGame -- Serializing narrative action map...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
	
	var narrativeActionMap = game.getNarrativeActionMap();
	for (var key in narrativeActionMap)
	{
		this.narrativeActionSaveMap[key] = new rp.NarrativeActionSave(narrativeActionMap[key]);
	}
	
	JSON.stringify(this);

	rpEngine.logger.log('RpEngine.newGame -- Serializing actor map...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

	var actorMap = game.getActorMap();
	for (var key in actorMap)
	{
		this.actorSaveMap[key] = new rp.ActorSave(actorMap[key]);
	}
	
	JSON.stringify(this);

	rpEngine.logger.log('RpEngine.newGame -- Serializing node map...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

	var nodeMap = game.getNodeMap();
	for (var key in nodeMap)
	{
		this.nodeSaveMap[key] = new rp.NodeSave(nodeMap[key]);
	}
	
	JSON.stringify(this);

	rpEngine.logger.log('RpEngine.newGame -- Serializing variable map...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

	this.variableMap = game.getVariableMap();

	rpEngine.logger.log('RpEngine.newGame -- Serializing text token lists...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

	var currentGameTextTokenList = game.getCurrentGameTextTokenList();
	for (var key in currentGameTextTokenList)
	{
		var textToken = currentGameTextTokenList[key];
		if (rp.TextTokenLang.prototype.isPrototypeOf(textToken))
		{
			this.currentGameTextTokenSaveList.push(new rp.TextTokenLangSave(textToken));
		}
		else if (rp.TextTokenRaw.prototype.isPrototypeOf(textToken))
		{
			this.currentGameTextTokenSaveList.push(new rp.TextTokenRawSave(textToken));
		}
		else
		{
			rpEngine.logger.log('GameSave -- Error, unknown text token type.', [], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
		}
	}
	
	JSON.stringify(this);

	var currentInfoTextTokenList = game.getCurrentInfoTextTokenList();
	for (var key in currentInfoTextTokenList)
	{
		var textToken = currentInfoTextTokenList[key];
		if (rp.TextTokenLang.prototype.isPrototypeOf(textToken))
		{
			this.currentInfoTextTokenSaveList.push(new rp.TextTokenLangSave(textToken));
		}
		else if (rp.TextTokenRaw.prototype.isPrototypeOf(textToken))
		{
			this.currentInfoTextTokenSaveList.push(new rp.TextTokenRawSave(textToken));
		}
		else
		{
			rpEngine.logger.log('GameSave -- Error, unknown text token type.', [], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
		}
	}
	
	JSON.stringify(this);

	rpEngine.logger.log('RpEngine.newGame -- Serializing general game data...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

	this.currentNodeId = ((game.getCurrentNode() != null) ? game.getCurrentNode().getId() : '');
	this.currentGameImageId = game.getCurrentGameImageId();
	this.playerActorId = ((game.getPlayerActor() != null) ? game.getPlayerActor().getId() : '');
	
	JSON.stringify(this);

	rpEngine.logger.log('RpEngine.newGame -- Serializing sound engine data...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
	
	this.soundEngineSave = new rp.SoundEngineSave(rpEngine.soundEngine);
	
	JSON.stringify(this);

	rpEngine.logger.log('RpEngine.newGame -- Finished serializing save data.', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

GameSave.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	currentModuleId : '',
	version : '',

	actorSaveMap : null,
	nodeSaveMap : null,
	narrativeActionSaveMap : null,
	variableMap : null,

	currentNodeId : '',
	currentGameImageId : '',
	currentGameTextTokenSaveList : null,
	currentInfoTextTokenSaveList : null,
	
	playerActorId : '',
	
	soundEngineSave : null,

	//------------------------------------------------------------------------------------------------------------
	//INITIALIZERS - SERIALIZERS
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = GameSave;

//------------------------------------------------------------------------------------------------------------