//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function Module()
{
	this._actorStatCategoryMap = {};
	this._actorStatTemplateMap = {};
	this._globalScriptMap = {};
	this._inventoryItemCategoryMap = {};
	this._inventoryItemTemplateMap = {};
	this._nodeTemplateMap = {};
	this._timeUnitTemplateMap = {};
	
	this._defaultDataMap = {};

	this._assetFilePathList = [];
	this._gameDataFilePathList = [];
	this._langFilePathList = [];
	
	this._gameDataObjectList = [];
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

Module.FILE_ROLE_VALUE = 'module';
Module.FILE_ROLE_VALUE_GAME_DATA = 'gameData';

Module.DEFAULT_DATA_NODE_ID_NEW_GAME = 'nodeIdNewGame';
Module.DEFAULT_DATA_TITLE_FONT_ID = 'titleFontId';
Module.DEFAULT_DATA_TITLE_IMAGE_ID = 'titleImageId';
Module.DEFAULT_DATA_TITLE_MUSIC_ID = 'titleMusicId';

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

Module.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_id : "",
	_moduleDirectoryPath : "",

	//Données de jeu
	
	_actorStatCategoryMap : null,
	_actorStatTemplateMap : null,
	_globalScriptMap : null,
	_inventoryItemCategoryMap : null,
	_inventoryItemTemplateMap : null,
	_nodeTemplateMap : null,
	_timeUnitTemplateMap : null,
	
	_defaultDataMap : null,

	//Fichiers de données
	
	_assetFilePathList : null,
	_gameDataFilePathList : null,
	_langFilePathList : null,
	
	_gameDataObjectList : null,
	
	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	formatSubFilePath : function(filePath)
	{
		return this._moduleDirectoryPath + filePath;
	},
	
	loadData : function()
	{
		rpModule = this;
		rpGlobalScriptMap = {};
		
		//Chargement des données générales
		
		rpEngine.logger.log('Loading general game data...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		for (var index in this._gameDataObjectList)
		{
			var dataList = this._gameDataObjectList[index].dataList;
			if (dataList != null)
			{
				for (var key in dataList)
				{
					this._defaultDataMap[key] = dataList[key];
				}
			}
		}
		
		//Chargement des scripts globaux

		rpEngine.logger.log('Loading global scripts...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		for (var index in this._gameDataObjectList)
		{
			var globalScriptList = this._gameDataObjectList[index].globalScriptList;
			if (globalScriptList != null)
			{
				for (var key in globalScriptList)
				{
					var globalScriptClass = rpEngine.assetManager.getPackGame().getScriptData(globalScriptList[key]);
					var globalScript = new globalScriptClass();
					
					this._globalScriptMap[key] = globalScript;
					rpGlobalScriptMap[key] = globalScript;
				}
			}
		}
		
		//Chargement des modèles de nodes

		rpEngine.logger.log('Loading node templates...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		for (var index in this._gameDataObjectList)
		{
			var nodeTemplateList = this._gameDataObjectList[index].nodeTemplateList;
			if (nodeTemplateList != null)
			{
				for (var key in nodeTemplateList)
				{
					var nodeTemplate = new rp.NodeTemplate(key);
					nodeTemplate.initFromFile(nodeTemplateList[key]);
					
					this._nodeTemplateMap[key] = nodeTemplate;
				}
			}
		}

		//Chargement des catégories de stats des acteurs

		rpEngine.logger.log('Loading actor stat categories...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		for (var index in this._gameDataObjectList)
		{
			var actorStatCategoryList = this._gameDataObjectList[index].actorStatCategoryList;
			if (actorStatCategoryList != null)
			{
				for (var key in actorStatCategoryList)
				{
					var actorStatCategory = new rp.ActorStatCategory(key);
					actorStatCategory.initFromFile(actorStatCategoryList[key]);
					
					this._actorStatCategoryMap[key] = actorStatCategory;
				}
			}
		}
		
		//Chargement des modèles de stats des acteurs

		rpEngine.logger.log('Loading actor stat templates...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

		for (var index in this._gameDataObjectList)
		{
			var actorStatTemplateList = this._gameDataObjectList[index].actorStatTemplateList;
			if (actorStatTemplateList != null)
			{
				for (var key in actorStatTemplateList)
				{
					var actorStatTemplate = new rp.ActorStatTemplate(key);
					actorStatTemplate.initFromFile(actorStatTemplateList[key]);
					
					this._actorStatTemplateMap[key] = actorStatTemplate;
				}
			}
		}
		
		//Assignation des effets aux modèles de stats des acteurs

		rpEngine.logger.log('Loading actor stat templates\' effects...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		for (var key in this._actorStatTemplateMap)
		{
			this._actorStatTemplateMap[key].initActorEffectList();
		}

		//Chargement des catégories d'items

		rpEngine.logger.log('Loading inventory item categories...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		for (var index in this._gameDataObjectList)
		{
			var inventoryItemCategoryList = this._gameDataObjectList[index].inventoryItemCategoryList;
			if (inventoryItemCategoryList != null)
			{
				for (var key in inventoryItemCategoryList)
				{
					var inventoryItemCategory = new rp.InventoryItemCategory(key);
					inventoryItemCategory.initFromFile(inventoryItemCategoryList[key]);
					
					this._inventoryItemCategoryMap[key] = inventoryItemCategory;
				}
			}
		}

		//Chargement des modèles d'items

		rpEngine.logger.log('Loading inventory item templates...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);

		for (var index in this._gameDataObjectList)
		{
			var inventoryItemTemplateList = this._gameDataObjectList[index].inventoryItemTemplateList;
			if (inventoryItemTemplateList != null)
			{
				for (var key in inventoryItemTemplateList)
				{
					var inventoryItemTemplate = new rp.InventoryItemTemplate(key);
					inventoryItemTemplate.initFromFile(inventoryItemTemplateList[key]);
					
					this._inventoryItemTemplateMap[key] = inventoryItemTemplate;
				}
			}
		}
		
		//Assignation des effets aux modèles d'items

		rpEngine.logger.log('Loading inventory item templates\' effects...', [], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		for (var key in this._inventoryItemTemplateMap)
		{
			this._inventoryItemTemplateMap[key].initActorEffectList();
		}

		/*

		//Chargement des catégories de stats des acteurs

		for (GameDataFileData data : dataList)
		{
			for (Entry<String, GameActorStatCategoryFileData> subData : data.getActorStatCategoryList().entrySet())
			{
				GameActorStatCategory object = new GameActorStatCategory(subData.getKey(), subData.getValue());
				this._actorStatCategoryMap.put(subData.getKey(), object);
			}
		}

		//Chargement des modèles de stats des acteurs

		Map<String, GameActorStatTemplateFileData> tempActorStatDataMap = new HashMap<>();
		
		for (GameDataFileData data : dataList)
		{
			for (Entry<String, GameActorStatTemplateFileData> subData : data.getActorStatTemplateList().entrySet())
			{
				tempActorStatDataMap.put(subData.getKey(), subData.getValue());
				
				GameActorStatTemplate object = new GameActorStatTemplate(subData.getKey());
				this._actorStatTemplateMap.put(subData.getKey(), object);
			}
		}
		
		for (Iterator<Entry<String, GameActorStatTemplate>> it = this._actorStatTemplateMap.entrySet().iterator(); it.hasNext();)
		{
			Entry<String, GameActorStatTemplate> object = it.next();

			boolean loadResult = object.getValue().load(tempActorStatDataMap.get(object.getKey()));
			if (loadResult == true)
			{
				object.getValue().getStatCategory().getStatTemplateMap().put(object.getKey(), object.getValue());
			}
			else
			{
				RP_Engine.logger.log(LogSubject.UNDEFINED, LogLevel.ERROR, "Module.loadDataGameDataList -- Cannot load actor stat template with ID '%s'.", true, object.getKey());
				
				it.remove();
			}
		}

		//Chargement des catégories d'items

		for (GameDataFileData data : dataList)
		{
			for (Entry<String, GameInventoryItemCategoryFileData> subData : data.getInventoryItemCategoryList().entrySet())
			{
				GameInventoryItemCategory object = new GameInventoryItemCategory(subData.getKey(), subData.getValue());
				this._inventoryItemCategoryMap.put(subData.getKey(), object);
			}
		}

		//Chargement des modèles d'items

		for (GameDataFileData data : dataList)
		{
			for (Entry<String, GameInventoryItemTemplateFileData> subData : data.getInventoryItemTemplateList().entrySet())
			{
				GameInventoryItemTemplate object = new GameInventoryItemTemplate(subData.getKey());
				boolean loadResult = object.load(subData.getValue());
				
				if (loadResult == true)
				{
					object.getItemCategory().getItemTemplateMap().put(object.getId(), object);
				}
				else
				{
					RP_Engine.logger.log(LogSubject.UNDEFINED, LogLevel.ERROR, "Module.loadDataGameDataList -- Cannot load inventory item template with ID '%s'.", true, object.getId());
				}
				this._inventoryItemTemplateMap.put(subData.getKey(), object);
			}
		}
		
		//Chargement des modèles d'unités de temps
		
		for (GameDataFileData data : dataList)
		{
			for (Entry<String, TimeUnitTemplateFileData> subData : data.getTimeUnitTemplateList().entrySet())
			{
				TimeUnitTemplate object = new TimeUnitTemplate(subData.getKey(), subData.getValue());
				this._timeUnitTemplateMap.put(subData.getKey(), object);
			}
		}
		*/
	},
	
	loadFile : function(directoryPath)
	{
		this._moduleDirectoryPath = directoryPath;
		var filePath = appBasePath + this._moduleDirectoryPath + 'base.json';
		
		var data = rp.Util.loadFileJson(filePath);
		if (data == null)
		{
			rpEngine.logger.log('Module.loadFile -- Could not load module file "%s".', [filePath], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
			
			return false;
		}

		//Vérification des erreurs
		
		if (data.role != rp.Module.FILE_ROLE_VALUE)
		{
			rpEngine.logger.log('Module.loadFile -- Bad role ID for module file "%s".', [filePath], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
		}
		
		//Chargement des chemins des fichiers
		
		this._assetFilePathList = data.assetFilePathList;
		this._gameDataFilePathList = data.gameDataFilePathList;
		this._langFilePathList = data.langFilePathList;

		for (var key in this._assetFilePathList)
		{
			rpEngine.assetManager.getPackGame().loadFile(this.formatSubFilePath(this._assetFilePathList[key]));
		}

		for (var key in this._langFilePathList)
		{
			rpEngine.langManager.getPackGame().loadFile(this.formatSubFilePath(this._langFilePathList[key]));
		}

		for (var key in this._gameDataFilePathList)
		{
			var gameDataObject = this.loadFileGameData(this.formatSubFilePath(this._gameDataFilePathList[key]));
			if (gameDataObject != null)
			{
				this._gameDataObjectList.push(gameDataObject);
			}
		}
		
		this.loadData();
	},
	
	loadFileGameData : function(filePath)
	{
		rpEngine.logger.log('Loading game data file "%s"...', [filePath], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_ENGINE);
		
		var filePathReal = appBasePath + filePath;
		var fileData = rp.Util.loadFileJson(filePathReal);
		if (fileData == null)
		{
			rpEngine.logger.log('GameData.loadFile -- Could not load game data file "%s".', [filePath], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
			return null;
		}

		//Vérification des erreurs
		
		if (fileData.role != Module.FILE_ROLE_VALUE_GAME_DATA)
		{
			rpEngine.logger.log('GameData.loadFile -- Bad role ID for game data file "%s".', [filePath], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
			return null;
		}
		
		//Chargement des données
		
		return fileData;
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------

	getActorStatCategory : function(id)
	{
		return this._actorStatCategoryMap[id];
	},
	getActorStatCategoryMap : function()
	{
		return this._actorStatCategoryMap;
	},
	
	getActorStatTemplate : function(id)
	{
		return this._actorStatTemplateMap[id];
	},
	getActorStatTemplateMap : function()
	{
		return this._actorStatTemplateMap;
	},
	
	getDefaultData : function(id)
	{
		return this._defaultDataMap[id];
	},
	getDefaultDataMap : function()
	{
		return this._defaultDataMap;
	},
	
	getGlobalScript : function(id)
	{
		return this._globalScriptMap[id];
	},
	getGlobalScriptMap : function()
	{
		return this._globalScriptMap;
	},
	
	getInventoryItemCategory : function(id)
	{
		return this._inventoryItemCategoryMap[id];
	},
	getInventoryItemCategoryMap : function()
	{
		return this._inventoryItemCategoryMap;
	},

	getInventoryItemTemplate : function(id)
	{
		return this._inventoryItemTemplateMap[id];
	},
	getInventoryItemTemplateMap : function()
	{
		return this._inventoryItemTemplateMap;
	},
	
	getModuleDirectoryPath : function()
	{
		return this._moduleDirectoryPath;
	},
	
	getNodeTemplate : function(id)
	{
		return this._nodeTemplateMap[id];
	},
	getNodeTemplateMap : function()
	{
		return this._nodeTemplateMap;
	},
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = Module;

//------------------------------------------------------------------------------------------------------------