//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function GameScreenMainPanel()
{
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

GameScreenMainPanel.CLASS_NARRATIVE_ACTION = 'game_narrativeAction';
GameScreenMainPanel.CLASS_NARRATIVE_ACTION_DESCRIPTION = 'game_narrativeAction_description';
GameScreenMainPanel.CLASS_NARRATIVE_ACTION_INFO = 'game_narrativeAction_info';
GameScreenMainPanel.CLASS_NARRATIVE_ACTION_TITLE = 'game_narrativeAction_title';
GameScreenMainPanel.ID_NARRATIVE_ACTION_LIST = 'game_narrativeActionList';

GameScreenMainPanel.CLASS_NARRATIVE_ACTION_SELECTOR = '.' + GameScreenMainPanel.CLASS_NARRATIVE_ACTION;
GameScreenMainPanel.CLASS_NARRATIVE_ACTION_DESCRIPTION_SELECTOR = '.' + GameScreenMainPanel.CLASS_NARRATIVE_ACTION_DESCRIPTION;
GameScreenMainPanel.CLASS_NARRATIVE_ACTION_INFO_SELECTOR = '.' + GameScreenMainPanel.CLASS_NARRATIVE_ACTION_INFO;
GameScreenMainPanel.CLASS_NARRATIVE_ACTION_TITLE_SELECTOR = '.' + GameScreenMainPanel.CLASS_NARRATIVE_ACTION_TITLE;
GameScreenMainPanel.ID_NARRATIVE_ACTION_LIST_SELECTOR = '#' + GameScreenMainPanel.ID_NARRATIVE_ACTION_LIST;

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

GameScreenMainPanel.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	_addNarrativeAction : function(narrativeAction)
	{
		var htmlSelectorNarrativeAction = GameScreenMainPanel.CLASS_NARRATIVE_ACTION_SELECTOR + '[data-id="' + narrativeAction.getId() + '"]';
		if ($(htmlSelectorNarrativeAction).length > 0)
		{
			rpEngine.logger.log('GameScreenMainPanel._addNarrativeAction -- Narrative action "%s" already exists.', [narrativeAction.getId()], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}

		var textDescription = rpEngine.langManager.getPackGame().getString(narrativeAction.getLangDescriptionId());
		var textInfo = rpEngine.langManager.getPackGame().getString(narrativeAction.getLangInfoId());
		var textTitle = rpEngine.langManager.getPackGame().getString(narrativeAction.getLangTitleId());

		var html =
				'<div data-id="' + narrativeAction.getId() + '" class="' + GameScreenMainPanel.CLASS_NARRATIVE_ACTION + '">' +
					'<div>' + 
						'<div class="' + GameScreenMainPanel.CLASS_NARRATIVE_ACTION_TITLE + '">' + textTitle + '</div>' +
						((narrativeAction.getLangDescriptionId() != '') ? '<div class="' + GameScreenMainPanel.CLASS_NARRATIVE_ACTION_DESCRIPTION + '">' + textDescription + '</div>' : '') +
						((narrativeAction.getLangInfoId() != '') ? '<div class="' + GameScreenMainPanel.CLASS_NARRATIVE_ACTION_INFO + '">' + textInfo + '</div>' : '') +
					'</div>' +
				'</div>';

		$(GameScreenMainPanel.ID_NARRATIVE_ACTION_LIST_SELECTOR).append(html);
		
		$(htmlSelectorNarrativeAction).bind('click', {'panel' : this}, function(event)
		{
			event.data.panel.onClickNarrativeAction(event);
		});
	},
	
	clearNarrativeActionList : function()
	{
		$(GameScreenMainPanel.ID_NARRATIVE_ACTION_LIST_SELECTOR).empty();
	},
	
	updateGameImage : function()
	{
		var gameImageId = rpEngine.game.getCurrentGameImageId();
		if (gameImageId != '')
		{
			var gameImagePath = rpEngine.moduleManager.formatCurrentModuleFilePathUrl(rpEngine.assetManager.getPackGame().getImagePath(gameImageId));
			$('#gameBackground').css('background-image', 'url("' + gameImagePath + '")');
		}
		else
		{
			$('#gameBackground').css('background-image', '');
		}
	},
	
	updateGameText : function()
	{
		var text = '';
		var gameTextTokenList = rpEngine.game.getCurrentGameTextTokenList()
		
		for (var key in gameTextTokenList)
		{
			text += gameTextTokenList[key].getValue();
		}
		
		var infoTextTokenList = rpEngine.game.getCurrentInfoTextTokenList()
		if (infoTextTokenList.length > 0)
		{
			var infoText = '';
			for (var key in infoTextTokenList)
			{
				infoText += ((infoText != '') ? '<br/>' : '') + infoTextTokenList[key].getValue();
			}
			
			infoText = '<hr/><p><em>' + infoText + '</em></p>';
			text += infoText;
		}
		
		$('#gameText').html(text);
	},

	updateNarrativeActionList : function()
	{
		$(GameScreenMainPanel.ID_NARRATIVE_ACTION_LIST_SELECTOR).empty();
		
		var narrativeActionList = rpEngine.game.getNarrativeActionMap();
		for (var key in narrativeActionList)
		{
			this._addNarrativeAction(narrativeActionList[key]);
		}
	},

	//------------------------------------------------------------------------------------------------------------
	//GUI EVENTS
	//------------------------------------------------------------------------------------------------------------

	onClickNarrativeAction : function(event)
	{
		rpEngine.game.executeNarrativeAction($(event.currentTarget).data('id'));
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------
		
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = GameScreenMainPanel;

//------------------------------------------------------------------------------------------------------------
