//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function InventoryPanel()
{
	this._inventoryPanelSub1 = new rp.InventoryPanelSub(this, rp.InventoryPanelSub.ID_INVENTORY_PANEL_SUB_1);
	this._inventoryPanelSub2 = new rp.InventoryPanelSub(this, rp.InventoryPanelSub.ID_INVENTORY_PANEL_SUB_2);

	$(InventoryPanel.ID_BTN_EQUIP_SELECTOR).bind('click', {'panel' : this}, function(event)
	{
		event.data.panel.onClickBtnEquip(event);
	});
	$(InventoryPanel.ID_BTN_UNEQUIP_SELECTOR).bind('click', {'panel' : this}, function(event)
	{
		event.data.panel.onClickBtnUnequip(event);
	});
	
	this.updateSelectedItemInfo();
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

InventoryPanel.ID_BTN_EQUIP = 'game_inventoryPanel_btnEquip';
InventoryPanel.ID_BTN_UNEQUIP = 'game_inventoryPanel_btnUnequip';
InventoryPanel.ID_SELECTED_ITEM_DESCRIPTION = 'game_inventoryPanel_selectedItemDescription';
InventoryPanel.ID_SELECTED_ITEM_INFO = 'game_inventoryPanel_selectedItemInfo';
InventoryPanel.ID_SELECTED_ITEM_NAME = 'game_inventoryPanel_selectedItemName';

InventoryPanel.ID_BTN_EQUIP_SELECTOR = '#' + InventoryPanel.ID_BTN_EQUIP;
InventoryPanel.ID_BTN_UNEQUIP_SELECTOR = '#' + InventoryPanel.ID_BTN_UNEQUIP;
InventoryPanel.ID_SELECTED_ITEM_DESCRIPTION_SELECTOR = '#' + InventoryPanel.ID_SELECTED_ITEM_DESCRIPTION;
InventoryPanel.ID_SELECTED_ITEM_INFO_SELECTOR = '#' + InventoryPanel.ID_SELECTED_ITEM_INFO;
InventoryPanel.ID_SELECTED_ITEM_NAME_SELECTOR = '#' + InventoryPanel.ID_SELECTED_ITEM_NAME;

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

InventoryPanel.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_inventoryPanelSub1 : null,
	_inventoryPanelSub2 : null,
	
	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	updateSelectedItemInfo : function()
	{
		var inventoryItemTemplateId = null;
		var isEquipped = false;
		
		var htmlSelectedInventoryItem = $(rp.InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_SELECTED_SELECTOR);
		if (htmlSelectedInventoryItem.length > 0)
		{
			inventoryItemTemplateId = htmlSelectedInventoryItem.data('id');
			isEquipped = htmlSelectedInventoryItem.hasClass(rp.InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_EQUIPPED);
		}
		
		//Initialisation des informations
		
		var textName = '';
		var textDescription = '';
		var textInfo = '';

		var showButtonEquip = false;
		var showButtonUnequip = false;
		var showButtonUse = false;
		
		if (inventoryItemTemplateId != null)
		{
			var inventoryItemTemplate = rpModule.getInventoryItemTemplate(inventoryItemTemplateId);
			if (inventoryItemTemplate != null)
			{			
				textName = inventoryItemTemplate.getLangName();
				textDescription = inventoryItemTemplate.getLangDescription();
				textInfo = inventoryItemTemplate.getLangInfoId();

				showButtonEquip = ((isEquipped == false) && (inventoryItemTemplate.getIsEquippable() == true));
				showButtonUnequip = (isEquipped == true);
			}
			else
			{
				rpEngine.logger.log('InventoryPanel.updateSelectedItemInfo -- Inventory item template "%s" does not exist.', [inventoryItemTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			}
		}
		
		//Mise à jour des textes
		
		$(rp.InventoryPanel.ID_SELECTED_ITEM_NAME_SELECTOR).html(textName);
		$(rp.InventoryPanel.ID_SELECTED_ITEM_DESCRIPTION_SELECTOR).html(textDescription);
		$(rp.InventoryPanel.ID_SELECTED_ITEM_INFO_SELECTOR).html(textInfo);
		
		//Mise à jour des boutons
		
		if (showButtonEquip == true)
		{
			$(rp.InventoryPanel.ID_BTN_EQUIP_SELECTOR).show();
		}
		else
		{
			$(rp.InventoryPanel.ID_BTN_EQUIP_SELECTOR).hide();
		}
		
		if (showButtonUnequip == true)
		{
			$(rp.InventoryPanel.ID_BTN_UNEQUIP_SELECTOR).show();
		}
		else
		{
			$(rp.InventoryPanel.ID_BTN_UNEQUIP_SELECTOR).hide();
		}
	},

	//------------------------------------------------------------------------------------------------------------
	//GUI EVENTS
	//------------------------------------------------------------------------------------------------------------

	onClickBtnEquip : function(event)
	{
		var htmlInventoryItem = $(rp.InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_SELECTED_SELECTOR);
		if (htmlInventoryItem.length === 0)
		{
			rpEngine.logger.log('InventoryPanel.onClickBtnEquip -- No inventory item selected.', [], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		var inventoryItemTemplateId = htmlInventoryItem.data('id');
		
		this._inventoryPanelSub1.getInventory().equipItem(inventoryItemTemplateId, 1, {});
	},

	onClickBtnUnequip : function(event)
	{
		var htmlInventoryItem = $(rp.InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_SELECTED_SELECTOR);
		if (htmlInventoryItem.length === 0)
		{
			rpEngine.logger.log('InventoryPanel.onClickBtnUnequip -- No inventory item selected.', [], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		var inventoryItemTemplateId = htmlInventoryItem.data('id');
		
		this._inventoryPanelSub1.getInventory().unequipItem(inventoryItemTemplateId, 1, {});
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------
	
	getInventoryPanelSub1 : function()
	{
		return this._inventoryPanelSub1;
	},

	getInventoryPanelSub2 : function()
	{
		return this._inventoryPanelSub2;
	},
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = InventoryPanel;

//------------------------------------------------------------------------------------------------------------
