//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function InventoryPanelSub(parentInventoryPanel, idInventoryPanelSub)
{
	this._parentInventoryPanel = parentInventoryPanel;
	
	this._idInventoryPanelSub = idInventoryPanelSub;
	this._idInventoryPanelSubSelector = '#' + idInventoryPanelSub;

	this._classInventoryItemSelector = this._idInventoryPanelSubSelector + ' .' + InventoryPanelSub.CLASS_INVENTORY_ITEM;
	this._classInventoryItemCategorySelector = this._idInventoryPanelSubSelector + ' .' + InventoryPanelSub.CLASS_INVENTORY_ITEM_CATEGORY;
	this._classInventoryItemCategoryNameSelector = this._idInventoryPanelSubSelector + ' .' + InventoryPanelSub.CLASS_INVENTORY_ITEM_CATEGORY_NAME;
	this._classInventoryItemCountSelector = this._classInventoryItemSelector + ' .' + InventoryPanelSub.CLASS_INVENTORY_ITEM_COUNT;
	this._classInventoryItemIsEquippedSelector = this._classInventoryItemSelector + '.' + InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_EQUIPPED;
	this._classInventoryItemIsUnequippedSelector = this._classInventoryItemSelector + ':not(.' + InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_EQUIPPED + ')';
	this._classInventoryItemListSelector = this._idInventoryPanelSubSelector + ' .' + InventoryPanelSub.CLASS_INVENTORY_ITEM_LIST;
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

InventoryPanelSub.ID_INVENTORY_PANEL_SUB_1 = 'game_inventoryPanel_sub1';
InventoryPanelSub.ID_INVENTORY_PANEL_SUB_2 = 'game_inventoryPanel_sub2';

InventoryPanelSub.CLASS_INVENTORY_ITEM = 'item';
InventoryPanelSub.CLASS_INVENTORY_ITEM_CATEGORY = 'itemCategory';
InventoryPanelSub.CLASS_INVENTORY_ITEM_CATEGORY_NAME = 'itemCategoryName';
InventoryPanelSub.CLASS_INVENTORY_ITEM_COUNT = 'count';
InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_EQUIPPED = 'isEquipped';
InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_SELECTED = 'isSelected';
InventoryPanelSub.CLASS_INVENTORY_ITEM_LIST = 'itemList';
InventoryPanelSub.CLASS_INVENTORY_NAME = 'inventoryName';

InventoryPanelSub.CLASS_INVENTORY_ITEM_SELECTOR = '.' + InventoryPanelSub.CLASS_INVENTORY_ITEM;
InventoryPanelSub.CLASS_INVENTORY_ITEM_CATEGORY_SELECTOR = '.' + InventoryPanelSub.CLASS_INVENTORY_ITEM_CATEGORY;
InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_SELECTED_SELECTOR = InventoryPanelSub.CLASS_INVENTORY_ITEM_SELECTOR + '.' + InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_SELECTED;

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

InventoryPanelSub.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_parentInventoryPanel : '',
	
	_idInventoryPanelSub : '',
	_idInventoryPanelSubSelector : '',
	
	_classInventoryItemSelector : '',
	_classInventoryItemCategorySelector : '',
	_classInventoryItemCategoryNameSelector : '',
	_classInventoryItemCountSelector : '',
	_classInventoryItemIsEquippedSelector : '',
	_classInventoryItemListSelector : '',
		
	_langPanelNameId : '',
	
	_inventory : null,
		
	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	_addInventoryItem : function(inventoryItem, isEquipped, sortList)
	{
		var inventoryItemTemplate = inventoryItem.getInventoryItemTemplate();
		var inventoryItemCategory = inventoryItemTemplate.getInventoryItemCategory();

		var htmlSelectorInventoryItem =
				((isEquipped == true) ? this._classInventoryItemIsEquippedSelector : this._classInventoryItemIsUnequippedSelector) + '[data-id="' + inventoryItemTemplate.getId() + '"]';
		var htmlSelectorInventoryItemCategory = this._classInventoryItemCategorySelector + '[data-id="' + inventoryItemCategory.getId() + '"]';
		
		if ($(htmlSelectorInventoryItem).length > 0)
		{
			rpEngine.logger.log('InventoryPanelSub._addInventoryItem -- Inventory item "%s" already exists.', [inventoryItemTemplate.getId()], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}

		var htmlInventoryItemCategory = $(htmlSelectorInventoryItemCategory);
		if (htmlInventoryItemCategory.length === 0)
		{
			this._addInventoryItemCategory(inventoryItemCategory, sortList);
			htmlInventoryItemCategory = $(htmlSelectorInventoryItemCategory);
		}
		
		var itemCount = ((isEquipped == true) ? inventoryItem.getItemCountEquipped() : inventoryItem.getItemCountUnequipped());
		var imagePath = 
				((inventoryItemTemplate.getImageId() != '') ?
				rpEngine.moduleManager.formatCurrentModuleFilePathUrl(rpEngine.assetManager.getPackGame().getImagePath(inventoryItemTemplate.getImageId())) :
				null);
		var imageCss = ((imagePath != null) ? 'background-image : url(\'' + imagePath + '\')"' : '');
		
		var html =
				'<div data-id="' + inventoryItemTemplate.getId() + '" ' +
						'class="' + InventoryPanelSub.CLASS_INVENTORY_ITEM + ' ' + ((isEquipped == true) ? InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_EQUIPPED : '') + '" ' +
						'style="' + imageCss + '">' +
					'<span class="' + InventoryPanelSub.CLASS_INVENTORY_ITEM_COUNT + '">' + itemCount + '</span>' +
				'</div>';

		htmlInventoryItemCategory.append(html);
		
		$(htmlSelectorInventoryItem).bind('click', {'panel' : this}, function(event)
		{
			event.data.panel.onClickInventoryItem(event);
		});

		if (sortList == true)
		{
			this._sortInventoryItemList(htmlSelectorInventoryItemCategory);
		}
	},

	_addInventoryItemCategory : function(inventoryItemCategory, sortList)
	{
		var htmlSelectorInventoryItemCategory = this._classInventoryItemCategorySelector + '[data-id="' + inventoryItemCategory.getId() + '"]';
		if ($(htmlSelectorInventoryItemCategory).length > 0)
		{
			rpEngine.logger.log(
					'InventoryPanelSub._addInventoryItemCategory -- Inventory item category "%s" already exists.', [inventoryItemCategory.getId()], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		var textInventoryItemCategoryName = rpEngine.langManager.getPackGame().getString(inventoryItemCategory.getLangNameId());
		
		var html =
				'<div data-id="' + inventoryItemCategory.getId() + '" class="' + InventoryPanelSub.CLASS_INVENTORY_ITEM_CATEGORY + '">' +
					'<div class="' + InventoryPanelSub.CLASS_INVENTORY_ITEM_CATEGORY_NAME + '">' + textInventoryItemCategoryName + '</div>' +
				'</div>';
		
		$(this._classInventoryItemListSelector).append(html);
		
		if (sortList == true)
		{
			this._sortInventoryItemCategoryList();
		}
	},

	_removeInventoryItem : function(inventoryItem, isEquipped)
	{
		var inventoryItemTemplate = inventoryItem.getInventoryItemTemplate();
		var inventoryItemCategory = inventoryItemTemplate.getInventoryItemCategory();
		
		var htmlSelectorInventoryItem =
			((isEquipped == true) ? this._classInventoryItemIsEquippedSelector : this._classInventoryItemIsUnequippedSelector) + '[data-id="' + inventoryItemTemplate.getId() + '"]';
		var htmlSelectorInventoryItemCategory = this._classInventoryItemCategorySelector + '[data-id="' + inventoryItemCategory.getId() + '"]';
		
		$(htmlSelectorInventoryItem).remove();
		
		if ($(htmlSelectorInventoryItemCategory).children(InventoryPanelSub.CLASS_INVENTORY_ITEM_SELECTOR).length === 0)
		{
			this._removeInventoryItemCategory(inventoryItemCategory);
		}
	},

	_removeInventoryItemCategory : function(inventoryItemCategory)
	{
		var htmlSelectorInventoryItemCategory = this._classInventoryItemCategorySelector + '[data-id="' + inventoryItemCategory.getId() + '"]';

		$(htmlSelectorInventoryItemCategory).remove();
	},

	_sortInventoryItemList : function(htmlSelector)
	{
		var htmlInventoryItemCategory = $(htmlSelector);

		var htmlInventoryItemList = htmlInventoryItemCategory.children(InventoryPanelSub.CLASS_INVENTORY_ITEM_SELECTOR);
		
		htmlInventoryItemList.sort(function(obj1, obj2)
		{
			var inventoryItemIsEquipped1 = $(obj1).hasClass(InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_EQUIPPED);
			var inventoryItemIsEquipped2 = $(obj2).hasClass(InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_EQUIPPED);
			
			if ((inventoryItemIsEquipped1 == true) && (inventoryItemIsEquipped2 == false))
			{
				return -1;
			}
			else if ((inventoryItemIsEquipped1 == false) && (inventoryItemIsEquipped2 == true))
			{
				return 1;
			}

			var inventoryItemTemplateId1 = $(obj1).data('id');
			var inventoryItemTemplateId2 = $(obj2).data('id');
				
			var inventoryItemTemplate1 = rpModule.getInventoryItemTemplate(inventoryItemTemplateId1);
			var inventoryItemTemplate2 = rpModule.getInventoryItemTemplate(inventoryItemTemplateId2);

			var id1 = inventoryItemTemplate1.getLangName(); 
			var id2 = inventoryItemTemplate2.getLangName(); 

			return id1.localeCompare(id2);
		});

		$.each(htmlInventoryItemList, function(index, item)
		{
			htmlInventoryItemCategory.append(item); 
		});
	},
	
	_sortInventoryItemCategoryList : function()
	{
		var htmlTop = $(this._classInventoryItemListSelector);
		var htmlInventoryItemCategoryList = htmlTop.children(InventoryPanelSub.CLASS_INVENTORY_ITEM_CATEGORY_SELECTOR);
		
		htmlInventoryItemCategoryList.sort(function(obj1, obj2)
		{
			var inventoryItemCategory1 = rpModule.getInventoryItemCategory($(obj1).data('id'));
			var inventoryItemCategory2 = rpModule.getInventoryItemCategory($(obj2).data('id'));
			
			var order1 = inventoryItemCategory1.getOrder();
			var order2 = inventoryItemCategory2.getOrder();
			
			if (order1 < order2)
			{
				return -1;
			}
			else if (order1 > order2)
			{
				return 1;
			}
			
			var id1 = inventoryItemCategory1.getLangName(); 
			var id2 = inventoryItemCategory2.getLangName(); 

			return id1.localeCompare(id2);
		});

		$.each(htmlInventoryItemCategoryList, function(index, item)
		{
			htmlTop.append(item); 
		});
	},

	updateInventoryItem : function(inventoryItem, sortList)
	{
		var htmlSelectorInventoryItemUnequipped = this._classInventoryItemIsUnequippedSelector + '[data-id="' + inventoryItem.getId() + '"]';
		
		var htmlInventoryItemUnequipped = $(htmlSelectorInventoryItemUnequipped);
		if ((htmlInventoryItemUnequipped.length === 0) && (inventoryItem.getItemCountUnequipped() > 0))
		{
			this._addInventoryItem(inventoryItem, false, sortList);
		}
		else if ((htmlInventoryItemUnequipped.length > 0) && (inventoryItem.getItemCountUnequipped() == 0))
		{
			this._removeInventoryItem(inventoryItem, false, sortList);
		}
		else if ((htmlInventoryItemUnequipped.length > 0) && (inventoryItem.getItemCountUnequipped() > 0))
		{
			htmlInventoryItemUnequipped.children(InventoryPanelSub.CLASS_INVENTORY_ITEM_COUNT_SELECTOR).html(inventoryItem.getItemCountUnequipped());
		}

		var htmlSelectorInventoryItemEquipped = this._classInventoryItemIsEquippedSelector + '[data-id="' + inventoryItem.getId() + '"]';
		var htmlInventoryItemEquipped = $(htmlSelectorInventoryItemEquipped);
		if ((htmlInventoryItemEquipped.length === 0) && (inventoryItem.getItemCountEquipped() > 0))
		{
			this._addInventoryItem(inventoryItem, true, sortList);
		}
		else if ((htmlInventoryItemEquipped.length > 0) && (inventoryItem.getItemCountEquipped() == 0))
		{
			this._removeInventoryItem(inventoryItem, true, sortList);
		}
		else if ((htmlInventoryItemEquipped.length > 0) && (inventoryItem.getItemCountEquipped() > 0))
		{
			htmlInventoryItemEquipped.children(InventoryPanelSub.CLASS_INVENTORY_ITEM_COUNT_SELECTOR).html(inventoryItem.getItemCountEquipped());
		}

		this._parentInventoryPanel.updateSelectedItemInfo();
	},

	updateInventoryItemList : function()
	{
		$(this._classInventoryItemListSelector).empty();
		
		if (this._inventory == null)
		{
			return;
		}
		
		var itemLists = this._inventory.getItemLists();
		
		for (var inventoryItemCategoryId in itemLists)
		{
			var htmlSelectorInventoryItemCategory = this._classInventoryItemCategorySelector + '[data-id="' + inventoryItemCategoryId + '"]';
			
			var itemList = itemLists[inventoryItemCategoryId];
			for (var i = 0; i < itemList.length; i++)
			{
				var inventoryItem = itemLists[inventoryItemCategoryId][i];

				this.updateInventoryItem(inventoryItem, false);
			}
			
			this._sortInventoryItemList(htmlSelectorInventoryItemCategory);
		}

		this._sortInventoryItemCategoryList();
	},

	//------------------------------------------------------------------------------------------------------------
	//GUI EVENTS
	//------------------------------------------------------------------------------------------------------------
	
	onClickInventoryItem : function(event)
	{
		if ($(event.currentTarget).hasClass(InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_SELECTED) == true)
		{
			$(event.currentTarget).removeClass(InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_SELECTED);
		}
		else
		{
			$(InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_SELECTED_SELECTOR).removeClass(InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_SELECTED);
			
			$(event.currentTarget).addClass(InventoryPanelSub.CLASS_INVENTORY_ITEM_IS_SELECTED);
		}

		this._parentInventoryPanel.updateSelectedItemInfo();
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------

	getInventory : function()
	{
		return this._inventory;
	},
	setInventory : function(inventory)
	{
		if (this._inventory != null)
		{
			this._inventory.unregisterPanel(this);
		}
		
		this._inventory = inventory;
		this._inventory.registerPanel(this);
		
		this.updateInventoryItemList();
	},
	
	getLangPanelId : function()
	{
		return this._langPanelNameId;
	},
	setLangPanelId : function(value)
	{
		this._langPanelId = value;
	}
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = InventoryPanelSub;

//------------------------------------------------------------------------------------------------------------
