//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function PlayerCharacterPanel()
{
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

PlayerCharacterPanel.CLASS_ACTOR_STAT = 'game_playerCharacterPanel_actorStat';
PlayerCharacterPanel.CLASS_ACTOR_STAT_CATEGORY = 'game_playerCharacterPanel_actorStatCategory';
PlayerCharacterPanel.CLASS_ACTOR_STAT_LEVEL = 'level';
PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTED = 'toggle';
PlayerCharacterPanel.ID_ACTOR_INFO_NAME = 'game_playerCharacterPanel_infoName';
PlayerCharacterPanel.ID_ACTOR_INFO_TITLE = 'game_playerCharacterPanel_infoTitle';
PlayerCharacterPanel.ID_ACTOR_STAT_LIST = 'game_playerCharacterPanel_actorStatList';
PlayerCharacterPanel.ID_ACTOR_PORTRAIT_IMAGE = 'game_playerCharacterPanel_image';
PlayerCharacterPanel.ID_SELECTED_STAT_DESCRIPTION = 'game_playerCharacterPanel_selectedStatDescription';
PlayerCharacterPanel.ID_SELECTED_STAT_INFO = 'game_playerCharacterPanel_selectedStatInfo';
PlayerCharacterPanel.ID_SELECTED_STAT_NAME = 'game_playerCharacterPanel_selectedStatName';

PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTOR = '.' + PlayerCharacterPanel.CLASS_ACTOR_STAT;
PlayerCharacterPanel.CLASS_ACTOR_STAT_CATEGORY_SELECTOR = '.' + PlayerCharacterPanel.CLASS_ACTOR_STAT_CATEGORY;
PlayerCharacterPanel.CLASS_ACTOR_STAT_LEVEL_SELECTOR = '.' + PlayerCharacterPanel.CLASS_ACTOR_STAT_LEVEL;
PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTED_SELECTOR = '.' + PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTED;
PlayerCharacterPanel.ID_ACTOR_INFO_NAME_SELECTOR = '#' + PlayerCharacterPanel.ID_ACTOR_INFO_NAME;
PlayerCharacterPanel.ID_ACTOR_INFO_TITLE_SELECTOR = '#' + PlayerCharacterPanel.ID_ACTOR_INFO_TITLE;
PlayerCharacterPanel.ID_ACTOR_STAT_LIST_SELECTOR = '#' + PlayerCharacterPanel.ID_ACTOR_STAT_LIST;
PlayerCharacterPanel.ID_ACTOR_PORTRAIT_IMAGE_SELECTOR = '#' + PlayerCharacterPanel.ID_ACTOR_PORTRAIT_IMAGE;
PlayerCharacterPanel.ID_SELECTED_STAT_DESCRIPTION_SELECTOR = '#' + PlayerCharacterPanel.ID_SELECTED_STAT_DESCRIPTION;
PlayerCharacterPanel.ID_SELECTED_STAT_INFO_SELECTOR = '#' + PlayerCharacterPanel.ID_SELECTED_STAT_INFO;
PlayerCharacterPanel.ID_SELECTED_STAT_NAME_SELECTOR = '#' + PlayerCharacterPanel.ID_SELECTED_STAT_NAME;

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

PlayerCharacterPanel.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_playerActor : null,
		
	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	_addActorStat : function(actorStat, sortList)
	{
		var actorStatTemplate = actorStat.getActorStatTemplate();
		var actorStatCategory = actorStatTemplate.getActorStatCategory();

		var htmlSelectorActorStat = PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTOR + '[data-id="' + actorStat.getId() + '"]';
		if ($(htmlSelectorActorStat).length > 0)
		{
			rpEngine.logger.log('PlayerCharacterPanel._addActorStat -- Actor stat "%s" already exists.', [actorStat.getId()], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		var textActorStatName = actorStatTemplate.getLangName();
		var textActorStatLevel = actorStat.getValueCurrentFinal();

		var html =
				'<a data-id="' + actorStat.getId() + '" class="' + PlayerCharacterPanel.CLASS_ACTOR_STAT + '">' +
					textActorStatName + ' <span class="level">' + textActorStatLevel + '</span>' +
				'</a>';

		var htmlSelectorActorStatCategory = PlayerCharacterPanel.CLASS_ACTOR_STAT_CATEGORY_SELECTOR + '[data-id="' + actorStatCategory.getId() + '"]';
		var htmlActorStatCategory = $(htmlSelectorActorStatCategory);
		if (htmlActorStatCategory.length === 0)
		{
			this._addActorStatCategory(actorStatCategory, sortList);
			htmlActorStatCategory = $(htmlSelectorActorStatCategory);
		}
		
		htmlActorStatCategory.append(html);
		
		$(htmlSelectorActorStat).bind('click', {'panel' : this}, function(event)
		{
			event.data.panel.onClickActorStat(event);
		});
		
		this.updateActorStat(actorStat);
		
		if (sortList == true)
		{
			this._sortActorStatList(htmlSelectorActorStatCategory);
		}
	},
	
	_addActorStatCategory : function(actorStatCategory, sortList)
	{
		var htmlSelectorActorStatCategory = PlayerCharacterPanel.CLASS_ACTOR_STAT_CATEGORY_SELECTOR + '[data-id="' + actorStatCategory.getId() + '"]';
		if ($(htmlSelectorActorStatCategory).length > 0)
		{
			rpEngine.logger.log('PlayerCharacterPanel._addActorStatCategory -- Actor stat category "%s" already exists.', [actorStatCategory.getId()], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		var textActorStatCategoryName = actorStatCategory.getLangName();
		
		var html =
				'<div data-id="' + actorStatCategory.getId() + '" class="' + PlayerCharacterPanel.CLASS_ACTOR_STAT_CATEGORY + '">' +
					'<div class="statsTitle">' + textActorStatCategoryName + '</div>' +
				'</div>';
		
		$(PlayerCharacterPanel.ID_ACTOR_STAT_LIST_SELECTOR).append(html);
		
		if (sortList == true)
		{
			this._sortActorStatCategoryList();
		}
	},
	
	_sortActorStatList : function(htmlSelector)
	{
		var htmlCategory = $(htmlSelector);

		var htmlStatList = htmlCategory.children(PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTOR);
		
		htmlStatList.sort(function(obj1, obj2)
		{
			var actorStatTemplate1 = rpModule.getActorStatTemplate($(obj1).data('id'));
			var actorStatTemplate2 = rpModule.getActorStatTemplate($(obj2).data('id'));
			
			var order1 = actorStatTemplate1.getOrder();
			var order2 = actorStatTemplate2.getOrder();

			if (order1 < order2)
			{
				return -1;
			}
			else if (order1 > order2)
			{
				return 1;
			}
			
			var id1 = actorStatTemplate1.getLangName(); 
			var id2 = actorStatTemplate2.getLangName(); 

			return id1.localeCompare(id2);
		});

		$.each(htmlStatList, function(index, item)
		{
			htmlCategory.append(item); 
		});
	},
	
	_sortActorStatCategoryList : function()
	{
		var htmlTop = $(PlayerCharacterPanel.ID_ACTOR_STAT_LIST_SELECTOR);
		
		var htmlCategoryList = htmlTop.children(PlayerCharacterPanel.CLASS_ACTOR_STAT_CATEGORY_SELECTOR);
		
		htmlCategoryList.sort(function(obj1, obj2)
		{
			var actorStatCategory1 = rpModule.getActorStatCategory($(obj1).data('id'));
			var actorStatCategory2 = rpModule.getActorStatCategory($(obj2).data('id'));
			
			var order1 = actorStatCategory1.getOrder();
			var order2 = actorStatCategory2.getOrder();

			if (order1 < order2)
			{
				return -1;
			}
			else if (order1 > order2)
			{
				return 1;
			}
			
			var id1 = actorStatCategory1.getLangName(); 
			var id2 = actorStatCategory2.getLangName(); 

			return id1.localeCompare(id2);
		});

		$.each(htmlCategoryList, function(index, item)
		{
			htmlTop.append(item); 
		});
	},

	_updateSelectedStatInfo : function()
	{
		var textName = '';
		var textDescription = '';
		var textInfo = '';
		
		var htmlActorStat = $(PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTOR + PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTED_SELECTOR);
		if (htmlActorStat.length > 0)
		{
			var actorStatId = htmlActorStat.data('id');
			var actorStat = this._playerActor.getStat(actorStatId);
			var actorStatTemplate = actorStat.getActorStatTemplate();
			
			textName = actorStatTemplate.getLangName();
			textDescription = actorStatTemplate.getLangDescription();
			textInfo = actorStatTemplate.getLangInfo();
		}
		
		$(PlayerCharacterPanel.ID_SELECTED_STAT_NAME_SELECTOR).html(textName);
		$(PlayerCharacterPanel.ID_SELECTED_STAT_DESCRIPTION_SELECTOR).html(textDescription);
		$(PlayerCharacterPanel.ID_SELECTED_STAT_INFO_SELECTOR).html(textInfo);
	},

	updateActorStat : function(actorStat)
	{
		var htmlSelectorActorStat = PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTOR + '[data-id="' + actorStat.getId() + '"]';
		var htmlActorStat = $(htmlSelectorActorStat);
		if (htmlActorStat.length === 0)
		{
			this._addActorStat(actorStat, true);
			htmlActorStat = $(htmlSelectorActorStat);
		}
		
		var valueString = actorStat.getValueCurrentFinal() + ((actorStat.getValueMaxFinal() != null) ? ' / ' + actorStat.getValueMaxFinal() : '');

		htmlActorStat.children(PlayerCharacterPanel.CLASS_ACTOR_STAT_LEVEL_SELECTOR).html(valueString);
	},

	updateActorStatList : function()
	{
		$(PlayerCharacterPanel.ID_ACTOR_STAT_LIST_SELECTOR).empty();
		
		if (this._playerActor == null)
		{
			return;
		}
		
		var statMaps = this._playerActor.getStatMaps()
		for (var actorStatCategoryId in statMaps)
		{
			var htmlSelectorActorStatCategory = PlayerCharacterPanel.CLASS_ACTOR_STAT_CATEGORY_SELECTOR + '[data-id="' + actorStatCategoryId + '"]';
			var htmlActorStatCategory = $(htmlSelectorActorStatCategory);
			
			if (htmlActorStatCategory.length === 0)
			{
				var actorStatCategory = rpModule.getActorStatCategory(actorStatCategoryId);
				if (actorStatCategory == null)
				{
					rpEngine.logger.log(
							'PlayerCharacterPanel.updateActorStatList -- Actor stat category "%s" does not exist.', [actorStatCategoryId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
					continue;
				}

				this._addActorStatCategory(actorStatCategory, false);
			}
			htmlActorStatCategory = $(htmlSelectorActorStatCategory);
			
			for (var actorStatId in statMaps[actorStatCategoryId])
			{
				var actorStat = statMaps[actorStatCategoryId][actorStatId];

				this._addActorStat(actorStat, false);
			}
			
			this._sortActorStatList(htmlSelectorActorStatCategory);
		}
		
		this._sortActorStatCategoryList();
	},

	updateAll : function()
	{
		this.updateImage();
		this.updateInfo();
		this.updateActorStatList();
		
		this._updateSelectedStatInfo();
	},
	
	updateImage : function()
	{
		var imageId = null;
		var imagePath = null;
		
		if (this._playerActor != null)
		{
			imageId = this._playerActor.getPortraitImageId();
			if (imageId != '')
			{
				imagePath = rpEngine.moduleManager.formatCurrentModuleFilePathUrl(rpEngine.assetManager.getPackGame().getImagePath(imageId));
			}
		}
		
		if (imagePath != null)
		{
			$(PlayerCharacterPanel.ID_ACTOR_PORTRAIT_IMAGE_SELECTOR).css('background-image', 'url("' + imagePath + '")');
		}
		else
		{
			$(PlayerCharacterPanel.ID_ACTOR_PORTRAIT_IMAGE_SELECTOR).css('background-image', '');
		}
	},
	
	updateInfo : function()
	{
		var textActorName = '';
		var textActorTitle = '';
		
		if (this._playerActor != null)
		{
			textActorName = this._playerActor.getName();
			textActorTitle = this._playerActor.getTitle();
		}
		
		$(PlayerCharacterPanel.ID_ACTOR_INFO_NAME_SELECTOR).text(textActorName);
		$(PlayerCharacterPanel.ID_ACTOR_INFO_TITLE_SELECTOR).text(textActorTitle);
	},

	//------------------------------------------------------------------------------------------------------------
	//GUI EVENTS
	//------------------------------------------------------------------------------------------------------------
	
	onClickActorStat : function(event)
	{
		if ($(event.currentTarget).hasClass(PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTED) == true)
		{
			$(event.currentTarget).removeClass(PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTED);
		}
		else
		{
			$(PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTOR + PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTED_SELECTOR).removeClass(PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTED);
			
			$(event.currentTarget).addClass(PlayerCharacterPanel.CLASS_ACTOR_STAT_SELECTED);
		}

		event.data.panel._updateSelectedStatInfo();
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------
	
	getPlayerActor : function()
	{
		return this._playerActor;
	},
	setPlayerActor : function(actor)
	{
		if (this._playerActor != null)
		{
			this._playerActor.unregisterPanel(this);
		}
		
		this._playerActor = actor;
		this._playerActor.registerPanel(this);
		
		this.updateAll();
	},
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = PlayerCharacterPanel;

//------------------------------------------------------------------------------------------------------------
