//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function PopupImageSelection()
{
	$(PopupImageSelection.ID_POPUP_IMAGE_SELECTION_BTN_CANCEL_SELECTOR).click(this.onClickBtnCancel);
	$(PopupImageSelection.ID_POPUP_IMAGE_SELECTION_BTN_CONFIRM_SELECTOR).click(this.onClickBtnConfirm);

	$(PopupImageSelection.ID_POPUP_IMAGE_SELECTION_INPUT_SELECTOR).bind('input', {'popup' : this}, function(event)
	{
		event.data.popup.onInputText(event);
	});
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE = 'image';
PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_IS_SELECTED = 'isSelected';
PopupImageSelection.ID_POPUP_IMAGE_SELECTION = 'game_popupImageSelection';
PopupImageSelection.ID_POPUP_IMAGE_SELECTION_BTN_CANCEL = 'game_popupImageSelection_btnCancel';
PopupImageSelection.ID_POPUP_IMAGE_SELECTION_BTN_CONFIRM = 'game_popupImageSelection_btnConfirm';
PopupImageSelection.ID_POPUP_IMAGE_SELECTION_IMAGE_LIST = 'game_popupImageSelection_imageList';
PopupImageSelection.ID_POPUP_IMAGE_SELECTION_MESSAGE = 'game_popupImageSelection_message';

PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_SELECTOR = '.' + PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE;
PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_IS_SELECTED_SELECTOR =
		PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_SELECTOR + '.' + PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_IS_SELECTED;
PopupImageSelection.ID_POPUP_IMAGE_SELECTION_SELECTOR = '#' + PopupImageSelection.ID_POPUP_IMAGE_SELECTION;
PopupImageSelection.ID_POPUP_IMAGE_SELECTION_BTN_CANCEL_SELECTOR = '#' + PopupImageSelection.ID_POPUP_IMAGE_SELECTION_BTN_CANCEL;
PopupImageSelection.ID_POPUP_IMAGE_SELECTION_BTN_CONFIRM_SELECTOR = '#' + PopupImageSelection.ID_POPUP_IMAGE_SELECTION_BTN_CONFIRM;
PopupImageSelection.ID_POPUP_IMAGE_SELECTION_IMAGE_LIST_SELECTOR = '#' + PopupImageSelection.ID_POPUP_IMAGE_SELECTION_IMAGE_LIST;
PopupImageSelection.ID_POPUP_IMAGE_SELECTION_MESSAGE_SELECTOR = '#' + PopupImageSelection.ID_POPUP_IMAGE_SELECTION_MESSAGE;

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

PopupImageSelection.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	_updateBtnConfirmDisabled : function()
	{
		var htmlSelectorSelectedItem =
				PopupImageSelection.ID_POPUP_IMAGE_SELECTION_SELECTOR + ' ' + PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_SELECTOR +
				PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_IS_SELECTED_SELECTOR;

		var disabled = ($(htmlSelectorSelectedItem).length === 0);
		$(PopupImageSelection.ID_POPUP_IMAGE_SELECTION_BTN_CONFIRM_SELECTOR).prop('disabled', disabled);
	},
	
	show : function(imageSelectionData)
	{
		var textMessage = rpEngine.langManager.getPackGame().getString(imageSelectionData.getLangMessageId());

		$(PopupImageSelection.ID_POPUP_IMAGE_SELECTION_MESSAGE_SELECTOR).html(textMessage);

		$(PopupImageSelection.ID_POPUP_IMAGE_SELECTION_IMAGE_LIST_SELECTOR).empty();
		
		var imageIdList = imageSelectionData.getImageIdList();
		for (var i = 0; i < imageIdList.length; i++)
		{
			var imageId = imageIdList[i]
			var imagePath = ((imageId != '') ? rpEngine.moduleManager.formatCurrentModuleFilePathUrl(rpEngine.assetManager.getPackGame().getImagePath(imageId)) : null);
			var imageCss = ((imagePath != null) ? 'background-image : url(\'' + imagePath + '\')"' : '');
			
			var html =
					'<div data-id="' + imageId + '" ' +
							'class="' + PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE + '" ' +
							'style="' + imageCss + '">' +
					'</div>';

			$(PopupImageSelection.ID_POPUP_IMAGE_SELECTION_IMAGE_LIST_SELECTOR).append(html);

			var htmlSelectorImage = PopupImageSelection.ID_POPUP_IMAGE_SELECTION_SELECTOR + ' ' + PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_SELECTOR + '[data-id="' + imageId + '"]';
			$(htmlSelectorImage).bind('click', {'panel' : this}, function(event)
			{
				event.data.panel.onClickImage(event);
			});
		}
		
		$(PopupImageSelection.ID_POPUP_IMAGE_SELECTION_SELECTOR).fadeIn();
		$('#popupShadow').fadeIn();
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------

	onClickBtnCancel : function()
	{
		$(PopupImageSelection.ID_POPUP_IMAGE_SELECTION_SELECTOR).fadeOut();
		$('#popupShadow').fadeOut();
		
		rpEngine.game.executeNarrativeActionInputCancel();
	},

	onClickBtnConfirm : function()
	{
		$(PopupImageSelection.ID_POPUP_IMAGE_SELECTION_SELECTOR).fadeOut();
		$('#popupShadow').fadeOut();

		var htmlSelectorSelectedItem =
				PopupImageSelection.ID_POPUP_IMAGE_SELECTION_SELECTOR + ' ' + PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_SELECTOR +
				PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_IS_SELECTED_SELECTOR;

		rpEngine.game.executeNarrativeActionInputImageSelectionConfirm($(htmlSelectorSelectedItem).data('id'));
	},

	onClickImage : function(event)
	{
		if ($(event.currentTarget).hasClass(PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_IS_SELECTED) == true)
		{
			$(event.currentTarget).removeClass(PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_IS_SELECTED);
		}
		else
		{
			$(PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_IS_SELECTED_SELECTOR).removeClass(PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_IS_SELECTED);
			
			$(event.currentTarget).addClass(PopupImageSelection.CLASS_POPUP_IMAGE_SELECTION_IMAGE_IS_SELECTED);
		}

		this._updateBtnConfirmDisabled();
	},
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = PopupImageSelection;

//------------------------------------------------------------------------------------------------------------
