//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function PopupTextInput()
{
	$(PopupTextInput.ID_POPUP_TEXT_INPUT_BTN_CANCEL_SELECTOR).click(this.onClickBtnCancel);
	$(PopupTextInput.ID_POPUP_TEXT_INPUT_BTN_CONFIRM_SELECTOR).click(this.onClickBtnConfirm);

	$(PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT_SELECTOR).bind('input', {'popup' : this}, function(event)
	{
		event.data.popup.onInputText(event);
	});
	
	$(PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT_SELECTOR).bind('keydown', {'popup' : this}, function(event)
	{
		event.data.popup.onKeyDown(event);
	});
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

PopupTextInput.ID_POPUP_TEXT_INPUT = 'game_popupTextInput';
PopupTextInput.ID_POPUP_TEXT_INPUT_BTN_CANCEL = 'game_popupTextInput_btnCancel';
PopupTextInput.ID_POPUP_TEXT_INPUT_BTN_CONFIRM = 'game_popupTextInput_btnConfirm';
PopupTextInput.ID_POPUP_TEXT_INPUT_INFO = 'game_popupTextInput_info';
PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT = 'game_popupTextInput_input';
PopupTextInput.ID_POPUP_TEXT_INPUT_MESSAGE = 'game_popupTextInput_message';

PopupTextInput.ID_POPUP_TEXT_INPUT_SELECTOR = '#' + PopupTextInput.ID_POPUP_TEXT_INPUT;
PopupTextInput.ID_POPUP_TEXT_INPUT_BTN_CANCEL_SELECTOR = '#' + PopupTextInput.ID_POPUP_TEXT_INPUT_BTN_CANCEL;
PopupTextInput.ID_POPUP_TEXT_INPUT_BTN_CONFIRM_SELECTOR = '#' + PopupTextInput.ID_POPUP_TEXT_INPUT_BTN_CONFIRM;
PopupTextInput.ID_POPUP_TEXT_INPUT_INFO_SELECTOR = '#' + PopupTextInput.ID_POPUP_TEXT_INPUT_INFO;
PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT_SELECTOR = '#' + PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT;
PopupTextInput.ID_POPUP_TEXT_INPUT_MESSAGE_SELECTOR = '#' + PopupTextInput.ID_POPUP_TEXT_INPUT_MESSAGE;

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

PopupTextInput.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//INITIALIZERS
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	_updateBtnConfirmDisabled : function()
	{
		var disabled = 
				(($(PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT_SELECTOR).val().length < $(PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT_SELECTOR).prop('minLength')) ||
				 ($(PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT_SELECTOR).val().length > $(PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT_SELECTOR).prop('maxLength')));
		$(PopupTextInput.ID_POPUP_TEXT_INPUT_BTN_CONFIRM_SELECTOR).prop('disabled', disabled);
	},
		
	show : function(textInputData)
	{
		var textMessage = rpEngine.langManager.getPackGame().getString(textInputData.getLangMessageId());
		var inputLengthMax = textInputData.getInputLengthMax();
		var inputLengthMin = textInputData.getInputLengthMin();
	
		if ((inputLengthMin > inputLengthMax) && (inputLengthMax > 0))
		{
			inputLengthMin = inputLengthMax;
		}

		var textInfo = '';
		if ((inputLengthMax > 0) && (inputLengthMin > 0))
		{
			textInfo = vsprintf('Your answer must be between %s and %s characters long.', [inputLengthMin, inputLengthMax]);
		}
		else if (inputLengthMax > 0)
		{
			textInfo = vsprintf('Your answer can be up to %s characters long.', [inputLengthMax]);	
		}
		else if (inputLengthMin > 0)
		{
			textInfo = vsprintf('Your answer must be at least %s characters long.', [inputLengthMin]);
		}

		if (inputLengthMax > 0)
		{
			$(PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT_SELECTOR).prop('maxLength', inputLengthMax);
		}
		else
		{
			$(PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT_SELECTOR).removeProp('maxLength');
		}

		if (inputLengthMin > 0)
		{
			$(PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT_SELECTOR).prop('minLength', inputLengthMin);
		}
		else
		{
			$(PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT_SELECTOR).removeProp('minLength');
		}
		
		$(PopupTextInput.ID_POPUP_TEXT_INPUT_MESSAGE_SELECTOR).html(textMessage);
		$(PopupTextInput.ID_POPUP_TEXT_INPUT_INFO_SELECTOR).html(textInfo);
		$(PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT_SELECTOR).val('');

		this._updateBtnConfirmDisabled();
		
		$(PopupTextInput.ID_POPUP_TEXT_INPUT_SELECTOR).fadeIn();
		$('#popupShadow').fadeIn();
	},

	//------------------------------------------------------------------------------------------------------------
	//GUI EVENTS
	//------------------------------------------------------------------------------------------------------------
	
	onClickBtnCancel : function()
	{
		$(PopupTextInput.ID_POPUP_TEXT_INPUT_SELECTOR).fadeOut();
		$('#popupShadow').fadeOut();
		
		rpEngine.game.executeNarrativeActionInputCancel();
	},

	onClickBtnConfirm : function()
	{
		if ($(PopupTextInput.ID_POPUP_TEXT_INPUT_BTN_CONFIRM_SELECTOR).prop('disabled') == true)
		{
			return;
		}
		
		$(PopupTextInput.ID_POPUP_TEXT_INPUT_SELECTOR).fadeOut();
		$('#popupShadow').fadeOut();

		rpEngine.game.executeNarrativeActionInputTextInputConfirm($(PopupTextInput.ID_POPUP_TEXT_INPUT_INPUT_SELECTOR).val());
	},
	
	onInputText : function(event)
	{
		this._updateBtnConfirmDisabled();
	},

	onKeyDown : function(event)
	{
		//Enter
		if (event.keyCode == 13)
		{
			this.onClickBtnConfirm();
		}
	},

	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------
		
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = PopupTextInput;

//------------------------------------------------------------------------------------------------------------
