//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function Util()
{
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

Util.createDirectory = function(path)
{
	try
	{
		fileSystem.mkdirSync(path);
	}
	catch (exception)
	{
		if (exception.code != 'EEXIST')
		{
			throw exception;
		}
	}
}

Util.generateUuid = function()
{
	return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c)
	{
	    var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
	    return v.toString(16);
	});
},

Util.getCurrentDatetime = function(formatReadable)
{
	var currentDate = new Date();
	var year = currentDate.getFullYear();
	var month = currentDate.getMonth() + 1; //January is 0!
	var day = currentDate.getDate();
	var hour = currentDate.getHours();
	var minute = currentDate.getMinutes();
	var second = currentDate.getSeconds();

	day = (day < 10) ? '0' + day : day;
	month = (month < 10) ? '0' + month : month;
	hour = (hour < 10) ? '0' + hour : hour;
	minute = (minute < 10) ? '0' + minute : minute;
	second = (second < 10) ? '0' + second : second;

	var string = '';
	if (formatReadable == true)
	{
		string = year + '-' + month + '-' + day + ' ' + hour + ':' + minute + ':' + second;
	}
	else
	{
		string = year + '_' + month + '_' + day + '_' + hour + '_' + minute + '_' + second;
	}

	return string;
}

Util.loadFile = function(filePath)
{
	var data = null;
	try
	{
		data = fileSystem.readFileSync(filePath, {'encoding' : 'utf8'});
	}
	catch (exception)
	{
		rpEngine.logger.log('Util.loadFile -- Error loading file "%s" -- %s', [filePath, exception.message], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
	}
	
	return data;
}

Util.loadFileJson = function(filePath)
{
	var dataString = Util.loadFile(filePath);
	if (dataString == null)
	{
		return null;
	}
	
	//TODO: Upgrader la librairie JSON pour ne pas avoir ce hack
	dataString = dataString.replace(/\t/g, '').replace(/\r/g, '').replace(/\n/g, '');
	
	var data = null;
	try
	{
		data = JSON5.parse(dataString);
	}
	catch (exception)
	{
		rpEngine.logger.log('Util.loadFileJson -- Error parsing JSON in file "%s" -- %s', [filePath, exception.message], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
	}
	return data;
}

Util.saveFile = function(filePath, dataString, append)
{
	try
	{
		if (append == false)
		{
			fileSystem.writeFileSync(filePath, dataString, {'encoding' : 'utf8'});
		}
		else
		{
			fileSystem.appendFileSync(filePath, dataString, {'encoding' : 'utf8'});
		}
	}
	catch (exception)
	{
		rpEngine.logger.log('Util.saveFile -- Error saving file "%s" -- %s', [filePath, exception.message], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_ENGINE);
		return false;
	}
	
	return true;
}

Util.saveFileJson = function(filePath, data)
{
	Util.saveFile(filePath, JSON.stringify(data), false);
}

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

Util.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------
		
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = Util;

//------------------------------------------------------------------------------------------------------------
