//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function GameUtil()
{
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

GameUtil.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//INITIALIZERS - SERIALIZERS
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	changePlayerActorStat : function(actorStatTemplateId, changeValue, isLevelIncrease, showMessage)
	{
		var actorStatTemplate = rpModule.getActorStatTemplate(actorStatTemplateId)
		if (actorStatTemplate == null)
		{
			rpEngine.logger.log('Global_GameUtil.changePlayerActorStat -- Actor stat template "%s" does not exist.', [actorStatTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		var actorStat = rpEngine.game.getPlayerActor().getStat(actorStatTemplateId);
		
		var currentValue = actorStat.getValueCurrentBase();
		var currentProgress = actorStat.getValueProgressCurrent();
		
		var newValue = currentValue;
		var newProgress = currentProgress;
		
		if (isLevelIncrease == true)
		{
			newValue += changeValue;
		}
		else
		{
			if (changeValue > 0)
			{	
				var targetProgress = ((currentValue >= 0) ? (currentValue + 1) : 1);
				newProgress += changeValue;
				while (newProgress >= targetProgress)
				{
					newValue++;
					newProgress -= targetProgress;
					targetProgress = ((newValue >= 0) ? (newValue + 1) : 1);
				}
			}
			else if (changeValue < 0)
			{
				var targetProgress = ((currentValue >= 0) ? (currentValue) : 1);
				newProgress += changeValue;
				while (newProgress < 0)
				{
					newValue--;
					newProgress += targetProgress;
					targetProgress = ((newValue >= 0) ? (newValue) : 1);
				}
			}
		}
		
		this.setPlayerActorStat(actorStatTemplateId, newValue, newProgress, showMessage);
	},

	changePlayerInventoryItem : function(inventoryItemTemplateId, itemCount, showMessage, options)
	{
		var inventoryItemTemplate = rpModule.getInventoryItemTemplate(inventoryItemTemplateId);
		if (inventoryItemTemplate == null)
		{
			rpEngine.logger.log(
					'Global_GameUtil.changePlayerInventoryItem -- Inventory item template "%s" does not exist.', [inventoryItemTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		var inventoryItemCategory = inventoryItemTemplate.getInventoryItemCategory();
		var inventoryItemCategoryName = rpEngine.langManager.getPackGame().getString(inventoryItemCategory.getLangNameId());
		var inventoryItemTemplateName = rpEngine.langManager.getPackGame().getString(inventoryItemTemplate.getLangNameId());

		var oldItemCount = rpEngine.game.getPlayerActor().getInventory().getItemCount(inventoryItemTemplateId, {});
		
		if (itemCount > 0)
		{
			rpEngine.game.getPlayerActor().getInventory().addItem(inventoryItemTemplateId, itemCount, options);

			var newItemCount = rpEngine.game.getPlayerActor().getInventory().getItemCount(inventoryItemTemplateId, {});
			if ((showMessage == true) && (oldItemCount != newItemCount))
			{
				//TODO: Remplacer les strings raw par des strings lang
				rpEngine.game.addTextInfoRaw('You have gained %s %s.', [(newItemCount - oldItemCount), inventoryItemTemplateName]);
			}
		}
		else if (itemCount < 0)
		{
			itemCount *= -1;
			
			rpEngine.game.getPlayerActor().getInventory().removeItem(inventoryItemTemplateId, itemCount, options);

			var newItemCount = rpEngine.game.getPlayerActor().getInventory().getItemCount(inventoryItemTemplateId, {});
			if ((showMessage == true) && (oldItemCount != newItemCount))
			{
				//TODO: Remplacer les strings raw par des strings lang
				rpEngine.game.addTextInfoRaw('You have lost %s %s.', [(oldItemCount - newItemCount), inventoryItemTemplateName]);
			}
		}
		else
		{
			rpEngine.logger.log(
					'Global_GameUtil.changePlayerInventoryItem -- "itemCount" is equal to 0.', [inventoryItemTemplateId], rp.Logger.LEVEL_WARNING, rp.Logger.SUBJECT_GAME);
		}
	},

	resetNodeContent : function()
	{
		game.clearGameText();
		game.clearGameActions();
		game.clearInfoText();
	},

	setActorStatToMax : function(actor, statId)
	{
		var actorStat = actor.getStat(statId);
		if (actorStat.getValueMaxFinal() === null)
		{
			var actorStatId = actorStat.getId();
			var actorId = ((actorStat.getParentActor() != null) ? actorStat.getParentActor().getId() : '');
			
			rpEngine.logger.log('Global_GameUtil.setActorStatToMax -- Stat "%s" for actor "%s" has no maximum.', [actorStatId, actorId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		actorStat.setValueCurrentBase(actorStat.getValueMaxFinal());
		actorStat.setValueProgressCurrent(0);
	},
	
	setPlayerActorStat : function(actorStatId, newValue, newProgress, showMessage)
	{
		var actorStat = rpEngine.game.getPlayerActor().getStat(actorStatId);
		var currentValue = actorStat.getValueCurrentBase();
		var currentProgress = actorStat.getValueProgressCurrent();

		if (showMessage == true)
		{
			var actorStatTemplateName = rpEngine.langManager.getPackGame().getString(actorStat.getActorStatTemplate().getLangNameId());
			var actorStatCategoryName = rpEngine.langManager.getPackGame().getString(actorStat.getActorStatCategory().getLangNameId());

			//TODO: Remplacer les strings raw par des strings lang
			if (newValue != currentValue)
			{
				if (newValue > currentValue)
				{
					rpEngine.game.addTextInfoRaw('Your %s - %s has increased by %s.', [actorStatCategoryName, actorStatTemplateName, (newValue - currentValue)]);
				}
				else if (newValue < currentValue)
				{
					rpEngine.game.addTextInfoRaw('Your %s - %s has decreased by %s.', [actorStatCategoryName, actorStatTemplateName, (currentValue - newValue)]);
				}
			}
			else if (newProgress != currentProgress)
			{
				if (newProgress > currentProgress)
				{
					rpEngine.game.addTextInfoRaw('Your %s - %s is increasing...', [actorStatCategoryName, actorStatTemplateName]);
				}
				else if (newProgress < currentProgress)
				{
					rpEngine.game.addTextInfoRaw('Your %s - %s is decreasing...', [actorStatCategoryName, actorStatTemplateName]);
				}
			}
		}
		
		actorStat.setValueCurrentBase(newValue);
		actorStat.setValueProgressCurrent(newProgress);
	},

	setPlayerActorStatListToMax : function(statIdList)
	{
		for (var i = 0; i < statIdList.length; i++)
		{
			this.setActorStatToMax(rpEngine.game.getPlayerActor(), statIdList[i]);
		}
	},

	setPlayerActorStatToMax : function(statId)
	{
		this.setActorStatToMax(rpEngine.game.getPlayerActor(), statId);
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------
		
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = GameUtil;

//------------------------------------------------------------------------------------------------------------
